"""Tests for log capture."""

import logging

import pytest

from llmdebug.log_capture import (
    get_recent_logs,
    install_log_handler,
    uninstall_log_handler,
)


@pytest.fixture(autouse=True)
def cleanup_handler():
    """Ensure handler is uninstalled after each test."""
    yield
    uninstall_log_handler()


def test_install_log_handler():
    """Test that log handler can be installed."""
    install_log_handler(max_records=10)

    # Log something
    logger = logging.getLogger("test")
    logger.setLevel(logging.DEBUG)
    logger.info("Test message")

    logs = get_recent_logs()
    assert len(logs) == 1
    assert logs[0]["message"] == "Test message"
    assert logs[0]["level"] == "INFO"


def test_get_recent_logs_returns_empty_without_handler():
    """Test that get_recent_logs returns empty list when no handler installed."""
    logs = get_recent_logs()
    assert logs == []


def test_log_capture_max_records():
    """Test that log records are limited to max_records."""
    install_log_handler(max_records=5)

    logger = logging.getLogger("test_max")
    logger.setLevel(logging.DEBUG)

    # Log more than max_records
    for i in range(10):
        logger.info(f"Message {i}")

    logs = get_recent_logs()
    assert len(logs) == 5

    # Should have the last 5 messages
    messages = [log["message"] for log in logs]
    assert messages == ["Message 5", "Message 6", "Message 7", "Message 8", "Message 9"]


def test_log_capture_with_limit_parameter():
    """Test that get_recent_logs respects the max_records parameter."""
    install_log_handler(max_records=10)

    logger = logging.getLogger("test_limit")
    logger.setLevel(logging.DEBUG)

    for i in range(10):
        logger.info(f"Message {i}")

    # Get only last 3
    logs = get_recent_logs(max_records=3)
    assert len(logs) == 3

    messages = [log["message"] for log in logs]
    assert messages == ["Message 7", "Message 8", "Message 9"]


def test_log_levels_captured():
    """Test that all log levels are captured."""
    install_log_handler()

    logger = logging.getLogger("test_levels")
    logger.setLevel(logging.DEBUG)

    logger.debug("Debug msg")
    logger.info("Info msg")
    logger.warning("Warning msg")
    logger.error("Error msg")

    logs = get_recent_logs()
    assert len(logs) == 4

    levels = [log["level"] for log in logs]
    assert levels == ["DEBUG", "INFO", "WARNING", "ERROR"]


def test_log_metadata_captured():
    """Test that log metadata is captured correctly."""
    install_log_handler()

    logger = logging.getLogger("test_meta")
    logger.setLevel(logging.DEBUG)
    logger.info("Test metadata")

    logs = get_recent_logs()
    assert len(logs) == 1

    log = logs[0]
    assert "timestamp" in log
    assert log["level"] == "INFO"
    assert log["logger"] == "test_meta"
    assert log["message"] == "Test metadata"
    assert "filename" in log
    assert "lineno" in log


def test_log_with_exception():
    """Test that exception info is captured in logs."""
    install_log_handler()

    logger = logging.getLogger("test_exc")
    logger.setLevel(logging.DEBUG)

    try:
        raise ValueError("test error")
    except ValueError:
        logger.exception("An error occurred")

    logs = get_recent_logs()
    assert len(logs) == 1
    assert logs[0]["exc_type"] == "ValueError"


def test_install_idempotent():
    """Test that installing multiple times is safe."""
    install_log_handler(max_records=5)
    install_log_handler(max_records=5)  # Should not create duplicate handlers

    logger = logging.getLogger("test_idem")
    logger.setLevel(logging.DEBUG)
    logger.info("Test")

    logs = get_recent_logs()
    # Should only have one message, not duplicated
    assert len(logs) == 1


def test_install_updates_max_records():
    """Test that reinstalling updates max_records."""
    install_log_handler(max_records=10)

    logger = logging.getLogger("test_update")
    logger.setLevel(logging.DEBUG)

    for i in range(10):
        logger.info(f"Message {i}")

    # Update to smaller max_records
    install_log_handler(max_records=5)

    # Should still have 5 messages (truncated to new max)
    logs = get_recent_logs()
    assert len(logs) == 5


def test_uninstall_clears_logs():
    """Test that uninstalling removes the handler."""
    install_log_handler()

    logger = logging.getLogger("test_uninstall")
    logger.setLevel(logging.DEBUG)
    logger.info("Before uninstall")

    uninstall_log_handler()

    # After uninstall, no logs should be returned
    logs = get_recent_logs()
    assert logs == []


def test_logs_oldest_first():
    """Test that logs are returned oldest first."""
    install_log_handler()

    logger = logging.getLogger("test_order")
    logger.setLevel(logging.DEBUG)

    logger.info("First")
    logger.info("Second")
    logger.info("Third")

    logs = get_recent_logs()
    messages = [log["message"] for log in logs]
    assert messages == ["First", "Second", "Third"]


def test_multiple_loggers():
    """Test that logs from multiple loggers are captured."""
    install_log_handler()

    logger1 = logging.getLogger("logger1")
    logger1.setLevel(logging.DEBUG)
    logger2 = logging.getLogger("logger2")
    logger2.setLevel(logging.DEBUG)

    logger1.info("From logger1")
    logger2.info("From logger2")

    logs = get_recent_logs()
    assert len(logs) == 2

    loggers = [log["logger"] for log in logs]
    assert "logger1" in loggers
    assert "logger2" in loggers



import math
from math import ceil

n, m = map(int, input().split())
ans = n**2 + 2

if m > n**2:
    print(-1)
    exit()

for i in range(1, 10**6 + 1):
    p = ceil(m / i)
    if 1 <= p <= n and m <= i * p <= n**2:
        ans = min(ans, i * p)
if ans == n**2 + 2:
    print(-1)
else:
    print(ans)



N, Q = map(int, input().split())
li = [i for i in range(N + 1)]
ind = [i for i in range(N + 1)]
for i in range(Q):
    A = int(input())
    B = ind[A]

    if B != N:
        temp = li[B]
        li[B] = li[B + 1]
        li[B + 1] = temp
        ind[A] = B + 1
        ind[li[B]] = B
    else:
        temp = li[B]
        li[B] = li[B - 1]
        li[B - 1] = temp
        ind[A] = B - 1
        ind[li[B]] = B

print(*li)



n, k = map(int, input().split())
s = input()

ans_list = []
cnt = 0
for i in range(n):
    if s[i] == "o":
        cnt += 1
        if cnt >= k:
            ans_list.append("x")
        else:
            ans_list.append("o")
    else:
        ans_list.append(s[i])
print(*ans_list, sep="")



import subprocess
import sys


def test_condefects_0():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='3 3 5\nS.G\no#o\n.#.\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '1\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_1():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='3 3 1\nS.G\n.#o\no#.\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '-1\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_2():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='5 10 2000000\nS.o..ooo..\n..o..o.o..\n..o..ooo..\n..o..o.o..\n..o..ooo.G\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '18\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_3():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='1 292 279\n.....................oS..............o...o.o........o............o.......o.......o...................o.......G.......o..o............o........................................................o.............o..................o.....o...................................o.........................o\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '14\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_4():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='1 298 391\n.o...............o...............o..................o....o....................o.............o..............................o...............o...............S...........................o.....o..................................G.....o...o....................o.............oo..............o.o..........\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '14\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )



from buggy import Solution


def test_case_0():
    sol = Solution()
    result = sol.sumSubarrayMins([3, 1, 2, 4])
    assert result == 17


def test_case_1():
    sol = Solution()
    result = sol.sumSubarrayMins([11, 81, 94, 43, 3])
    assert result == 444



n, w = map(int, input().split())
an = list(map(int, input().split()))

an.append(0)
an.append(0)
an.append(0)
ans = set()
n += 3
for i in range(n):
    for j in range(n):
        for k in range(n):
            total = an[i] + an[j] + an[k]
            if total != 0 and total < w and i != j and i != k and j != k:
                ans.add(total)
print(len(ans))



n = int(input())
s = list(input())
if "".join(s) == "AB":
    print("No")
    exit()
if s[0] == "B":
    print("Yes")
    exit()
if s[0] == "A" and s[-1] == "B":
    print("No")
    exit()
print("Yes")



from sys import stdin

readline = stdin.readline
t = int(readline())
mod = 998244353
for _ in range(t):
    n = int(readline())
    m = int(n**0.5 + 0.001)
    ans = pow(m, 3, mod)
    j = m
    for i in range(m, 0, -1):
        x = n // i
        ans += 3 * (x - j) * i**2
        ans %= mod
        j = x
    print(ans)



from collections import deque

N, M = map(int, input().split())
XY = list(set([tuple(map(int, input().split())) for _ in range(M)]))
to = [[] for _ in range(N + 1)]
deg = [0 for _ in range(N + 1)]
for X, Y in XY:
    if X == Y:
        continue
    to[X].append(Y)
    deg[Y] += 1

dq = deque([i for i in range(1, N + 1) if deg[i] == 0])

P = []
while dq:
    x = dq.popleft()
    P.append(x)
    for to_x in to[x]:
        deg[to_x] -= 1
        if deg[to_x] == 0:
            dq.append(to_x)
    if len(dq) >= 2:
        print("No")
        exit()

if len(P) != N:
    print("No")
    exit()

ans = [0 for _ in range(N + 1)]
for i in range(N):
    ans[P[i]] = i + 1

print("Yes")
print(*ans[1:])



def solve(h, w, m, t, a, x):
    a = [_ - 1 for _ in a]
    cnt = {color: 0 for color in x}  # 色ごとの塗られたマスの個数
    S_row = set()  # すでに塗られた行の集合
    S_col = set()  # すでに塗られた列の集合
    # 逆順に塗られたマスの数を調べていく
    for k in range(m - 1, -1, -1):
        if t[k] == 1:  # a[k]列目のマスを色x[k]で塗る
            if not a[k] in S_row:
                S_row.add(a[k])
                cnt[x[k]] += w - len(S_col)
        else:  # a[k]行目のマスを色x[k]で塗る
            if not a[k] in S_col:
                S_col.add(a[k])
                cnt[x[k]] += h - len(S_row)
    cnt[0] = h * w - sum(cnt.values())
    return {color: num for color, num in cnt.items() if num > 0}


h, w, m = map(int, input().split())
t, a, x = zip(*[map(int, input().split()) for i in range(m)])
color_num_mapping = solve(h, w, m, t, a, x)
print(len(color_num_mapping))
for color, num in sorted(color_num_mapping.items()):
    print(color, num)



import sys

# sys.setrecursionlimit(200005)
int1 = lambda x: int(x) - 1
p2D = lambda x: print(*x, sep="\n", end="\n\n")


def II():
    return int(sys.stdin.readline())


def LI():
    return list(map(int, sys.stdin.readline().split()))


def LLI(rows_number):
    return [LI() for _ in range(rows_number)]


def LI1():
    return list(map(int1, sys.stdin.readline().split()))


def LLI1(rows_number):
    return [LI1() for _ in range(rows_number)]


def SI():
    return sys.stdin.readline().rstrip()


# dij = [(0, 1), (-1, 0), (0, -1), (1, 0)]
# dij = [(0, 1), (-1, 0), (0, -1), (1, 0), (1, 1), (1, -1), (-1, 1), (-1, -1)]
inf = 18446744073709551615
# inf = 4294967295
md = 10**9 + 7
# md = 998244353

from collections import Counter


class Sieve:
    def __init__(self, n):
        self.plist = [2]  # n以下の素数のリスト
        min_prime_factor = [2, 0] * (n // 2 + 1)
        for x in range(3, n + 1, 2):
            if min_prime_factor[x] == 0:
                min_prime_factor[x] = x
                self.plist.append(x)
                if x**2 > n:
                    continue
                for y in range(x**2, n + 1, 2 * x):
                    if min_prime_factor[y] == 0:
                        min_prime_factor[y] = x
        self.min_prime_factor = min_prime_factor

    def isprime(self, x):
        return self.min_prime_factor[x] == x

    # これが素因数分解(prime factorization)
    def pfct(self, x):
        pp, ee = [], []
        while x > 1:
            mpf = self.min_prime_factor[x]
            if pp and mpf == pp[-1]:
                ee[-1] += 1
            else:
                pp.append(mpf)
                ee.append(1)
            x //= mpf
        return pp, ee


mx = 200005
sv = Sieve(mx)
mu = [-1] * mx
mu[1] = 0
for p in sv.plist:
    for a in range(p, mx, p):
        mu[a] = -mu[a]
    for a in range(p * p, mx, p * p):
        mu[a] = 0
# print(mu)

n = II()
pp = [-1] + LI()


def fac(a):
    pp, _ = sv.pfct(a)
    res = [1]
    for p in pp:
        nr = []
        for s in res:
            nr.append(s * p)
        res += nr
    return res[1:]


def cnt_pair(a):
    if a * 2 > n:
        return 1
    cnt = Counter()
    for i in range(a, n + 1, a):
        # print(a, pp[i], fac(pp[i]))
        for f in fac(pp[i]):
            cnt[f] += 1
    s = 0
    for f, c in cnt.items():
        s += c * (c + 1) // 2 * mu[f]
    return s


ans = 0
for a in range(2, n + 1):
    if mu[a] == 0:
        continue
    ans += mu[a] * cnt_pair(a)
    # print(a, cnt_pair(a), ans)

print(ans)



import math
import sys

sys.setrecursionlimit(500_000)
from collections import defaultdict


def solve1(x, k, tmp, d):
    m = len(tmp)
    ans = 0 if len(tmp) == d + 2 else 1
    # print(f'ans={ans}, tmp={tmp}, d={d}')
    rest = tmp[m - 1] - x
    if rest < 0:
        return float("inf")
    j = m - 2
    while rest > 0:
        x = tmp[j]
        j -= 1
        if x <= rest:
            v = rest // x
            ans += v
            rest = rest % x
    return ans


def solve(d, k, x):
    d1 = 0
    tmp = [0]
    ans = float("inf")
    while tmp[-1] < x:
        tmp.append(tmp[-1] + k**d1)
        ans = min(ans, solve1(x, k, tmp, d))
        d1 += 1
    return ans


t = int(input())
for _ in range(t):
    d, k, x = list(map(int, input().split()))
    print(solve(d, k, x))



def neq(x):
    n = len(x)
    ans = 0
    if n == 1:
        return int(x)
    for i in range(n):
        if i == 0:
            ans += (int(x[i]) - 1) * (9 ** (n - 1)) + neq("9" * (n - 1))
        elif i == n - 1:
            if int(x[i - 1]) <= int(x[i]):
                return ans + int(x[i])
            else:
                return ans + int(x[i]) + 1
        elif int(x[i - 1]) < int(x[i]):
            ans += (int(x[i]) - 1) * (9 ** (n - 1 - i))
        elif x[i - 1] == x[i]:
            ans += int(x[i]) * (9 ** (n - 1 - i))
            return ans
        else:
            ans += (int(x[i])) * (9 ** (n - 1 - i))


T = int(input())
for _ in range(T):
    k = int(input())
    M = k * 10
    s, l = k, M
    while True:
        m = (s + l) // 2
        x = neq(str(m))
        if x < k:
            s = m
        elif x > k:
            l = m
        else:
            m = str(m)
            while True:
                for j in range(0, len(m) - 1):
                    if m[j] == m[j + 1]:
                        if j + 2 < len(m) - 1:
                            m = str(int(m[: j + 2]) - 1) + "9" * (len(m) - 1 - j - 2)
                        else:
                            m = str(int(m) - 1)
                        break
                else:
                    print(int(m))
                    break
            break



from buggy import Solution


def test_case_0():
    sol = Solution()
    result = sol.minimumTotalPrice(
        4, [[0, 1], [1, 2], [1, 3]], [2, 2, 10, 6], [[0, 3], [2, 1], [2, 3]]
    )
    assert result == 23


def test_case_1():
    sol = Solution()
    result = sol.minimumTotalPrice(2, [[0, 1]], [2, 2], [[0, 0]])
    assert result == 1



n, m = map(int, input().split())
L = list(map(int, input().split()))
left, right = max(L), sum(L)
while left < right:
    mid = (left + right) // 2
    cur = 0
    level = 0
    # print("lr", left, right, mid)
    flag = False
    for i in range(n):
        # print("lc", level, cur, L[i])
        if L[i] > mid:
            left = mid + 1
            flag = True
            break
        if cur + L[i] > mid:
            level += 1
            cur = 0
        cur += L[i] + 1
        if level >= m:
            left = mid + 1
            flag = True
            break

    if not flag:
        right = mid
print(right)



n = int(input())
a = list(map(int, input().split()))
b = list(map(int, input().split()))

ans1 = 0
ans2 = 0

for i in range(n):
    if a[i] == b[i]:
        ans1 += 1

for i in range(n):
    for j in range(n):
        if a[i] == b[j]:
            ans2 += 1

print(ans1)
print(ans2)



N, M = input().split()
S = tuple(input())
T = tuple(input())

if S == T[: int(N)]:
    if S == T[int(M) - int(N) + 1 :]:
        print(0)
    else:
        print(1)
else:
    if S == T[int(M) - int(N) :]:
        print(2)
    else:
        print(3)



import subprocess
import sys


def test_condefects_0():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='939303495398480680 414726308268777718 398560589355042192\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '-1\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_1():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='472406250225767566 355301439148824169 69364786724614022\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '-1\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_2():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='691276305791624098 118273667410328141 800494871051190582\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '800494871051190582\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_3():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='107609441806336587 651693647045658413 145433729648106087\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '-1\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_4():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='139165842603530408 274295409420274222 150900603803714103\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '274295409420274222\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )



"""Jupyter/IPython integration for automatic crash capture.

Captures debug snapshots on cell errors with rich HTML display in notebooks.
Follows the hooks.py pattern: module-level state, idempotent install/uninstall,
rate limiting, PII redaction.

Usage:
    # In a notebook cell:
    %load_ext llmdebug

    # Or programmatically:
    import llmdebug
    llmdebug.load_jupyter()

    # After any cell error, a compact banner is shown.
    # Use magic commands for deeper analysis:
    %llmdebug              # Show latest snapshot
    %llmdebug hypothesize  # Generate debugging hypotheses
    %llmdebug diff         # Compare latest vs previous
    %llmdebug list         # List recent snapshots
"""

from __future__ import annotations

import html as _html_mod
import sys
import threading
from collections.abc import Callable
from typing import Any

from .config import SnapshotConfig
from .hooks import _build_production_config, _capture_safely, _RateLimiter
from .redaction_policy import RedactionProfile

# ---------------------------------------------------------------------------
# Module-level state (same pattern as hooks.py)
# ---------------------------------------------------------------------------

_installed = False
_lock = threading.Lock()
_original_showtraceback: Callable[..., Any] | None = None
_cfg: SnapshotConfig | None = None
_limiter: _RateLimiter | None = None
_ipython_ref: Any = None  # Weak reference isn't needed — IPython is a singleton

# ---------------------------------------------------------------------------
# Display helpers
# ---------------------------------------------------------------------------

_BANNER_CSS = (
    "border-left:4px solid #e74c3c; background:#fdf2f2; padding:8px 12px;"
    "margin:8px 0; font-family:monospace; font-size:13px; border-radius:0 4px 4px 0"
)

_FULL_CSS = (
    "border:1px solid #ddd; background:#fafafa; padding:12px;"
    "margin:8px 0; font-family:monospace; font-size:13px; border-radius:4px"
)


def _is_notebook(ipython: Any) -> bool:
    """Detect Jupyter notebook vs terminal IPython."""
    return type(ipython).__name__ == "ZMQInteractiveShell"


def _get_out_dir() -> str:
    """Return the configured output directory, defaulting to '.llmdebug'."""
    return _cfg.out_dir if _cfg else ".llmdebug"


def _display_html_or_text(
    ipython: Any,
    *,
    html_fn: Callable[[], str],
    text_fn: Callable[[], str],
) -> None:
    """Display HTML in notebooks, plain text in terminals."""
    if _is_notebook(ipython):
        try:
            from IPython.display import HTML, display

            display(HTML(html_fn()))
            return
        except ImportError:
            pass
    print(text_fn())


def _display_markdown_or_text(ipython: Any, text: str) -> None:
    """Display Markdown in notebooks, plain text in terminals."""
    if _is_notebook(ipython):
        try:
            from IPython.display import Markdown, display

            display(Markdown(text))
            return
        except ImportError:
            pass
    print(text)


def _exc_summary(snapshot: dict[str, Any]) -> tuple[str, str, str, int | str]:
    """Extract (exc_type, exc_message, crash_file, crash_line) from snapshot."""
    exc = snapshot.get("exception", {})
    exc_type = exc.get("type", "UnknownError")
    exc_msg = exc.get("message", "")
    frames = snapshot.get("frames") or [{}]
    crash_idx = snapshot.get("crash_frame_index", 0)
    cfi = crash_idx if isinstance(crash_idx, int) and 0 <= crash_idx < len(frames) else 0
    fr0 = frames[cfi] if frames else {}
    crash_file = fr0.get("file_rel", fr0.get("file", "?"))
    crash_line = fr0.get("line", "?")
    return exc_type, exc_msg, crash_file, crash_line


def _format_snapshot_banner_html(snapshot: dict[str, Any]) -> str:
    """Compact HTML banner (~4 lines) shown after every cell error."""
    exc_type, exc_msg, crash_file, crash_line = _exc_summary(snapshot)
    # Truncate long messages
    if len(exc_msg) > 120:
        exc_msg = exc_msg[:117] + "..."
    return (
        f'<div style="{_BANNER_CSS}">'
        f"<b>llmdebug</b> captured <b>{_esc(exc_type)}</b>"
        f" at {_esc(str(crash_file))}:{crash_line}<br>"
        f"<span style='color:#666'>{_esc(exc_msg)}</span><br>"
        f"<span style='color:#888; font-size:11px'>"
        f"Use <code>%llmdebug</code> to view full snapshot, "
        f"<code>%llmdebug hypothesize</code> for debugging hints</span>"
        f"</div>"
    )


def _format_snapshot_banner_text(snapshot: dict[str, Any]) -> str:
    """Plain text banner for terminal IPython."""
    exc_type, exc_msg, crash_file, crash_line = _exc_summary(snapshot)
    if len(exc_msg) > 100:
        exc_msg = exc_msg[:97] + "..."
    return (
        f"\n[llmdebug] captured {exc_type} at {crash_file}:{crash_line}\n"
        f"  {exc_msg}\n"
        f"  Use %llmdebug to view, %llmdebug hypothesize for hints\n"
    )


def _format_full_snapshot_html(snapshot: dict[str, Any]) -> str:
    """Rich HTML display for %llmdebug show in notebooks."""
    exc = snapshot.get("exception", {})
    exc_type = exc.get("type", "UnknownError")
    exc_msg = exc.get("message", "")
    category = exc.get("error_category", {})
    frames = snapshot.get("frames") or []
    crash_idx = snapshot.get("crash_frame_index", 0)

    parts = [
        f'<div style="{_FULL_CSS}">',
        f"<h3 style='margin:0 0 8px 0; color:#e74c3c'>{_esc(exc_type)}</h3>",
        f"<p style='margin:0 0 8px 0'>{_esc(exc_msg)}</p>",
    ]

    # Error category
    if category:
        cat_name = category.get("category", "")
        suggestion = category.get("suggestion", "")
        if cat_name:
            parts.append(
                f"<p style='margin:4px 0; color:#2980b9'><b>Category:</b> {_esc(cat_name)}</p>"
            )
        if suggestion:
            parts.append(
                f"<p style='margin:4px 0; color:#27ae60'><b>Suggestion:</b> {_esc(suggestion)}</p>"
            )

    # Crash frame
    if frames:
        cfi = crash_idx if isinstance(crash_idx, int) and 0 <= crash_idx < len(frames) else 0
        fr0 = frames[cfi]
        file_name = fr0.get("file_rel", fr0.get("file", "?"))
        line = fr0.get("line", "?")
        func = fr0.get("function", "?")
        code = fr0.get("code", "")
        parts.append("<hr style='border:none;border-top:1px solid #eee;margin:8px 0'>")
        parts.append(
            f"<p style='margin:4px 0'><b>Crash frame:</b> "
            f"{_esc(str(file_name))}:{line} in <code>{_esc(func)}</code></p>"
        )
        if code:
            parts.append(
                f"<pre style='background:#f5f5f5;padding:8px;border-radius:3px;"
                f"overflow-x:auto;margin:4px 0'>{_esc(code)}</pre>"
            )

        # Locals
        locals_data = fr0.get("locals", {})
        if isinstance(locals_data, dict) and locals_data:
            parts.append("<p style='margin:8px 0 4px 0'><b>Locals:</b></p>")
            parts.append("<table style='border-collapse:collapse;width:100%;font-size:12px'>")
            for var_name, var_val in list(locals_data.items())[:20]:
                val_str = _format_local_value(var_val)
                parts.append(
                    f"<tr><td style='padding:2px 8px 2px 0;color:#7f8c8d;"
                    f"vertical-align:top;white-space:nowrap'>{_esc(var_name)}</td>"
                    f"<td style='padding:2px 0'><code>{_esc(val_str)}</code></td></tr>"
                )
            parts.append("</table>")

    # Jupyter context
    jupyter_ctx = snapshot.get("jupyter")
    if jupyter_ctx:
        parts.append("<hr style='border:none;border-top:1px solid #eee;margin:8px 0'>")
        ec = jupyter_ctx.get("execution_count", "?")
        parts.append(f"<p style='margin:4px 0;color:#888;font-size:11px'>Cell In[{ec}]</p>")

    parts.append("</div>")
    return "\n".join(parts)


def _format_full_snapshot_text(snapshot: dict[str, Any]) -> str:
    """Plain text/markdown display for terminal IPython."""
    exc = snapshot.get("exception", {})
    exc_type = exc.get("type", "UnknownError")
    exc_msg = exc.get("message", "")
    category = exc.get("error_category", {})
    frames = snapshot.get("frames") or []
    crash_idx = snapshot.get("crash_frame_index", 0)

    lines = [
        f"## {exc_type}",
        f"{exc_msg}",
        "",
    ]

    if category:
        cat_name = category.get("category", "")
        suggestion = category.get("suggestion", "")
        if cat_name:
            lines.append(f"**Category:** {cat_name}")
        if suggestion:
            lines.append(f"**Suggestion:** {suggestion}")
        lines.append("")

    if frames:
        cfi = crash_idx if isinstance(crash_idx, int) and 0 <= crash_idx < len(frames) else 0
        fr0 = frames[cfi]
        file_name = fr0.get("file_rel", fr0.get("file", "?"))
        line = fr0.get("line", "?")
        func = fr0.get("function", "?")
        code = fr0.get("code", "")
        lines.append(f"**Crash frame:** {file_name}:{line} in `{func}`")
        if code:
            lines.append(f"```python\n{code}\n```")

        locals_data = fr0.get("locals", {})
        if isinstance(locals_data, dict) and locals_data:
            lines.append("\n**Locals:**")
            for var_name, var_val in list(locals_data.items())[:20]:
                val_str = _format_local_value(var_val)
                lines.append(f"  {var_name} = {val_str}")

    jupyter_ctx = snapshot.get("jupyter")
    if jupyter_ctx:
        ec = jupyter_ctx.get("execution_count", "?")
        lines.append(f"\n_Cell In[{ec}]_")

    return "\n".join(lines)


def _format_local_value(val: Any) -> str:
    """Format a local variable value for display."""
    if isinstance(val, dict):
        # Array-like objects
        for key in ("__array__", "__dataframe__", "__series__"):
            if key in val:
                shape = val.get("shape", "?")
                dtype = val.get("dtype", "?")
                return f"{val[key]} shape={shape} dtype={dtype}"
        # Truncated string
        if "__truncated__" in val:
            return str(val.get("value", val))
    return str(val)[:100]


def _esc(text: str) -> str:
    """Escape HTML entities (safe for both text content and attribute values)."""
    return _html_mod.escape(text)


# ---------------------------------------------------------------------------
# Core: showtraceback wrapper
# ---------------------------------------------------------------------------


def _make_wrapped_showtraceback(
    original: Callable[..., Any],
    cfg: SnapshotConfig,
    limiter: _RateLimiter,
    ipython: Any,
) -> Callable[..., Any]:
    """Create a wrapped showtraceback that captures snapshots after errors."""

    def wrapped_showtraceback(*args: Any, **kwargs: Any) -> None:
        # Always call the original first — user sees normal traceback
        original(*args, **kwargs)

        # Extract the current exception info
        _exc_type, exc_value, exc_tb = sys.exc_info()
        if exc_value is None or exc_tb is None:
            return

        # Don't capture intentional exits
        if isinstance(exc_value, (KeyboardInterrupt, SystemExit)):
            return

        # Capture + display must never break IPython
        try:
            extra: dict[str, Any] = {"jupyter": _extract_jupyter_context(ipython)}
            _capture_safely("jupyter_cell", exc_value, exc_tb, cfg, limiter, extra=extra)
            _display_snapshot_banner(ipython, cfg.out_dir)
        except Exception as capture_exc:
            # A broken safety net is worse than a noisy one (hooks.py principle)
            sys.stderr.write(
                f"llmdebug: jupyter capture/display failed: "
                f"{type(capture_exc).__name__}: {capture_exc}\n"
            )

    return wrapped_showtraceback


def _extract_jupyter_context(ipython: Any) -> dict[str, Any]:
    """Extract Jupyter-specific context from the IPython shell."""
    ctx: dict[str, Any] = {}

    try:
        ctx["execution_count"] = ipython.execution_count
    except AttributeError:
        pass

    try:
        # Get the most recent cell source
        hm = ipython.history_manager
        if hm is not None:
            history = list(hm.get_tail(1, raw=True))
            if history:
                _, _, cell_source = history[0]
                # Truncate long cells
                if isinstance(cell_source, str):
                    if len(cell_source) > 500:
                        cell_source = cell_source[:497] + "..."
                    ctx["cell_source"] = cell_source
    except Exception as e:
        ctx["cell_source_error"] = f"{type(e).__name__}: {e}"

    return ctx


def _display_snapshot_banner(ipython: Any, out_dir: str) -> None:
    """Display a compact snapshot banner after an error."""
    from .output import get_latest_snapshot

    snapshot = get_latest_snapshot(out_dir)
    if snapshot is None:
        return

    _display_html_or_text(
        ipython,
        html_fn=lambda: _format_snapshot_banner_html(snapshot),
        text_fn=lambda: _format_snapshot_banner_text(snapshot),
    )


# ---------------------------------------------------------------------------
# Magic commands
# ---------------------------------------------------------------------------


def _register_magics(ipython: Any) -> None:
    """Register the %llmdebug magic command. Imports IPython only when called."""
    from IPython.core.magic import Magics, line_magic, magics_class

    @magics_class
    class LLMDebugMagics(Magics):  # type: ignore[misc]
        @line_magic
        def llmdebug(self, line: str) -> None:  # type: ignore[override]
            """LLMDebug magic command. Usage: %llmdebug [subcommand]"""
            parts = line.strip().split(None, 1)
            subcmd = parts[0] if parts else "show"

            if subcmd == "show":
                self._cmd_show()
            elif subcmd == "list":
                self._cmd_list()
            elif subcmd == "hypothesize":
                self._cmd_hypothesize()
            elif subcmd == "diff":
                self._cmd_diff()
            elif subcmd == "install":
                install(ipython)
                print("[llmdebug] installed")
            elif subcmd == "uninstall":
                uninstall()
                print("[llmdebug] uninstalled")
            elif subcmd == "config":
                self._cmd_config()
            else:
                self._cmd_help()

        def _cmd_show(self) -> None:
            from .output import get_latest_snapshot

            snapshot = get_latest_snapshot(_get_out_dir())
            if snapshot is None:
                print("[llmdebug] No snapshots found.")
                return

            _display_html_or_text(
                self.shell,
                html_fn=lambda: _format_full_snapshot_html(snapshot),
                text_fn=lambda: _format_full_snapshot_text(snapshot),
            )

        def _cmd_list(self) -> None:
            from ._snapshot_loader import find_all_snapshots

            entries = find_all_snapshots(_get_out_dir())
            if not entries:
                print("[llmdebug] No snapshots found.")
                return

            def _html() -> str:
                rows = []
                for i, e in enumerate(entries[:20], 1):
                    rows.append(
                        f"<tr><td>#{i}</td>"
                        f"<td>{_esc(e['exc_type'])}</td>"
                        f"<td>{_esc(e['exc_message'])}</td>"
                        f"<td>{_esc(str(e['file']))}:{e['line']}</td></tr>"
                    )
                return (
                    "<table style='border-collapse:collapse;font-family:monospace;"
                    "font-size:12px'>"
                    "<tr style='border-bottom:1px solid #ddd'>"
                    "<th style='padding:4px 8px'>#</th>"
                    "<th style='padding:4px 8px'>Type</th>"
                    "<th style='padding:4px 8px'>Message</th>"
                    "<th style='padding:4px 8px'>Location</th></tr>" + "\n".join(rows) + "</table>"
                )

            def _text() -> str:
                lines = []
                for i, e in enumerate(entries[:20], 1):
                    lines.append(
                        f"  #{i}  {e['exc_type']}: {e['exc_message']}  ({e['file']}:{e['line']})"
                    )
                return "\n".join(lines)

            _display_html_or_text(
                self.shell,
                html_fn=_html,
                text_fn=_text,
            )

        def _cmd_hypothesize(self) -> None:
            from .hypotheses import format_hypotheses_text, generate_hypotheses
            from .output import get_latest_snapshot

            snapshot = get_latest_snapshot(_get_out_dir())
            if snapshot is None:
                print("[llmdebug] No snapshots found.")
                return

            hypotheses = generate_hypotheses(snapshot)
            text = format_hypotheses_text(hypotheses)
            _display_markdown_or_text(self.shell, text)

        def _cmd_diff(self) -> None:
            from ._snapshot_loader import find_all_snapshots, resolve_snapshot_ref
            from .snapshot_diff import diff_snapshots, format_diff_text

            out_dir = _get_out_dir()
            all_snaps = find_all_snapshots(out_dir)

            try:
                new = resolve_snapshot_ref("latest", out_dir, all_snaps)
                old = resolve_snapshot_ref("previous", out_dir, all_snaps)
            except FileNotFoundError as e:
                print(f"[llmdebug] {e}")
                return

            result = diff_snapshots(old, new)
            text = format_diff_text(result)
            _display_markdown_or_text(self.shell, text)

        def _cmd_config(self) -> None:
            if _cfg is None:
                print("[llmdebug] Not installed. Run %llmdebug install first.")
                return
            import dataclasses

            print("[llmdebug] Active configuration:")
            for f in dataclasses.fields(_cfg):
                val = getattr(_cfg, f.name)
                print(f"  {f.name} = {val!r}")

        def _cmd_help(self) -> None:
            print(
                "Usage: %llmdebug [command]\n"
                "\n"
                "Commands:\n"
                "  show         Show latest snapshot (default)\n"
                "  list         List recent snapshots\n"
                "  hypothesize  Generate debugging hypotheses\n"
                "  diff         Compare latest vs previous snapshot\n"
                "  install      Install capture hooks\n"
                "  uninstall    Remove capture hooks\n"
                "  config       Show active configuration"
            )

    ipython.register_magics(LLMDebugMagics)


# ---------------------------------------------------------------------------
# Public API
# ---------------------------------------------------------------------------


def install(
    ipython: Any = None,
    *,
    out_dir: str = ".llmdebug",
    max_captures_per_minute: int = 10,
    pii_safe: bool = True,
    redaction_profile: RedactionProfile | str | None = None,
    include_git: bool = False,
    include_array_stats: bool = True,
    lock_timeout: float = 1.0,
    debug: bool = False,
    **config_overrides: Any,
) -> None:
    """Install llmdebug into the current IPython session.

    Idempotent — safe to call multiple times.

    Args:
        ipython: IPython shell instance. Auto-detected if None.
        out_dir: Snapshot output directory.
        max_captures_per_minute: Rate limit per unique error signature.
        pii_safe: Enable built-in PII redaction patterns.
        redaction_profile: Built-in redaction profile override.
        include_git: Include git context (disabled by default).
        include_array_stats: Include array shape/stats (enabled for data science).
        lock_timeout: File lock timeout in seconds.
        debug: Print capture failures to stderr.
        **config_overrides: Additional SnapshotConfig fields.
    """
    global _installed, _original_showtraceback, _cfg, _limiter, _ipython_ref

    with _lock:
        if _installed:
            return

        if ipython is None:
            try:
                from IPython import get_ipython  # type: ignore[attr-defined]

                ipython = get_ipython()
            except ImportError:
                raise RuntimeError(
                    "IPython is not installed. Install with: pip install llmdebug[jupyter]"
                ) from None
            if ipython is None:
                raise RuntimeError(
                    "No active IPython session found. "
                    "Run this inside IPython or a Jupyter notebook."
                )

        cfg = _build_production_config(
            out_dir,
            pii_safe,
            redaction_profile=redaction_profile,
            include_git=include_git,
            include_array_stats=include_array_stats,
            lock_timeout=lock_timeout,
            debug=debug,
            **config_overrides,
        )

        limiter = _RateLimiter(max_captures_per_minute, 60.0)

        # Save original and wrap showtraceback
        original: Callable[..., Any] = ipython.showtraceback
        _original_showtraceback = original
        ipython.showtraceback = _make_wrapped_showtraceback(original, cfg, limiter, ipython)

        _cfg = cfg
        _limiter = limiter
        _ipython_ref = ipython

        # Register magic commands
        _register_magics(ipython)

        _installed = True


def uninstall() -> None:
    """Remove llmdebug from the current IPython session.

    Idempotent — safe to call multiple times.
    """
    global _installed, _original_showtraceback, _cfg, _limiter, _ipython_ref

    with _lock:
        if not _installed:
            return

        # Restore original showtraceback
        if _ipython_ref is not None and _original_showtraceback is not None:
            _ipython_ref.showtraceback = _original_showtraceback

        _original_showtraceback = None
        _cfg = None
        _limiter = None
        _ipython_ref = None
        _installed = False


# ---------------------------------------------------------------------------
# IPython extension API
# ---------------------------------------------------------------------------


def load_ipython_extension(ipython: Any) -> None:
    """Called by ``%load_ext llmdebug``."""
    install(ipython)


def unload_ipython_extension(ipython: Any) -> None:
    """Called by ``%unload_ext llmdebug``."""
    uninstall()



import subprocess
import sys


def test_condefects_0():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='3 125 175\n200 300 400\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '2\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_1():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='1 1 1\n2\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '1\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_2():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='5 123 456\n135 246 357 468 579\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '5\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_3():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='1 1000 1000\n2000\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '1\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_4():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='58 440 491\n931 1743 983 1646 870 1143 1559 661 915 1552 46 13 1939 755 1893 780 1141 299 1106 152 1711 1654 66 1401 1190 768 874 1847 1485 1498 1418 1425 1351 191 992 1819 1873 713 1219 932 1004 377 447 104 441 356 518 419 1933 778 537 762 1791 509 910 1535 1805 930\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '1\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )



import sys
from collections import Counter

read = sys.stdin.read
readline = sys.stdin.readline
readlines = sys.stdin.readlines


N = int(readline())
S = readline().rstrip()

C = Counter(S)
S = list(S)
alp = "abcdefghijklmnopqrstuvwxyz"

now = 0
left = 0
right = N - 1

while left < right:
    while C[alp[now]] == 0:
        now += 1

    if S[left] == alp[now]:
        left += 1
        C[alp[now]] -= 1
        continue

    if S[right] != alp[now]:
        right -= 1
        C[S[right]] -= 1
    else:
        S[left], S[right] = S[right], S[left]
        C[alp[now]] -= 1
        C[S[right]] -= 1
        left += 1
        right -= 1

print("".join(S))



from buggy import Solution


def test_case_0():
    sol = Solution()
    result = sol.longestPrefix("level")
    assert result == "l"


def test_case_1():
    sol = Solution()
    result = sol.longestPrefix("ababab")
    assert result == "abab"



import subprocess
import sys


def test_condefects_0():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='3\n3 3\n2 4\n6 2\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '3\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_1():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='1\n1 4\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '2\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_2():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='1\n1 1\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '0\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_3():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='1\n1 1000000000\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '500000000\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_4():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='26\n4979445 861648772\n3690081 433933447\n6190629 262703497\n1047202 971407775\n8894325 731963982\n2804784 450968417\n302156 982631932\n1735902 880895728\n3078537 707723857\n9330739 910286918\n2329211 404539679\n3238506 317063340\n2686568 773361868\n5660016 650287940\n9296263 462224593\n2601449 384836991\n1890310 576823355\n2177068 404011431\n8008580 954291757\n449218 155374934\n594328 164163676\n7829355 138996221\n1899080 353195922\n5531545 910748511\n34067 913575467\n338674 824284691\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '4814609082\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )



# coding: utf-8

from fractions import Fraction

from functools import partial

try:
    dummy = src
    minp = partial(src.pop, 0)
except NameError:
    minp = input


def ints():
    return list(map(int, minp().rstrip().split(" ")))


def int1():
    return int(minp().rstrip())


def above_below(p, p1, p2):
    """
    点(x, y)が２点(x1, y1),(x2, y2)を通る直線の上にあるか下にあるか
    0なら直線上、正なら+y領域、負なら-y領域
    """
    (x, y), (x1, y1), (x2, y2) = p, p1, p2

    # 鉛直線の場合
    if x1 == x2:
        return x - x1

    # 直線 a x + y + b = 0 を求める
    a = Fraction(-(y1 - y2) / (x1 - x2))
    b = -(a * x1 + y1)
    # x, yを代入して返す
    return a * x + y + b


# @psecs
def main():
    pts = [tuple(ints()) for _ in range(4)]
    #    print(locals())

    # 対角線が作る直線に対して残りの２点が反対側にあれば凸
    if (
        above_below(pts[0], pts[1], pts[3]) * above_below(pts[2], pts[1], pts[3]) < 0
        and above_below(pts[1], pts[0], pts[2]) * above_below(pts[3], pts[1], pts[2]) < 0
    ):
        print("Yes")
    else:
        print("No")


if __name__ == "__main__":
    main()



N, K = map(int, input().split())
A = sorted([int(x) for x in input().split()])
mod = 998244353
d = 1
from collections import Counter

C = Counter(A)
for c in C:
    for i in range(C[c]):
        d *= i + 1
        d %= mod
x = pow(d, mod - 2, mod)
t = 1
j = N - 1
for i in range(N):
    while j > i:
        if A[j] + A[i] >= K:
            t += 1
        else:
            break
        j -= 1
    if i == j:
        break
    x *= t * (t - 1)
    x %= mod
    t -= 1
for i in range(t + 1):
    x *= i + 1
    x %= mod
print(x)



from buggy import Solution


def test_case_0():
    sol = Solution()
    result = sol.averageValue([1,3,6,10,12,15])
    assert result == 9


def test_case_1():
    sol = Solution()
    result = sol.averageValue([1,2,4,7,10])
    assert result == 0



import heapq

N = int(input())
Vs = list()
Es = list()
inf = 10**10
for i in range(1, N):
    Vs.append([i, inf])
    Ai, Bi, Xi = map(int, input().split())
    Es.append([i + i, Ai, Xi, Bi])
Vs.append([N, inf])
Vs[0][1] = 0

U = []
heapq.heappush(U, (Vs[0][1], Vs[0]))
now = heapq.heappop(U)[1]
while now[0] != N:
    i = Es[now[0] - 1][0]
    A = Es[now[0] - 1][1]
    X = Es[now[0] - 1][2]
    B = Es[now[0] - 1][3]
    if Vs[now[0]][1] == inf:
        Vs[now[0]][1] = now[1] + A
        heapq.heappush(U, (Vs[now[0]][1], Vs[now[0]]))
    elif Vs[now[0]][1] > now[1] + A:
        Vs[now[0]][1] = now[1] + A
    if Vs[X - 1][1] == inf:
        Vs[X - 1][1] = now[1] + B
        heapq.heappush(U, (Vs[X - 1][1], Vs[X - 1]))
    elif Vs[X - 1][1] > now[1] + B:
        Vs[X - 1][1] = now[1] + B
    now = heapq.heappop(U)[1]

print(Vs[N - 1][1])



n, a = list(map(int, input().split()))
T = list(map(int, input().split()))
time = 0
for i in T:
    if i >= time:
        time = i + a - time
        print(time)
    else:
        time += a
        print(time)



from buggy import Solution


def test_case_0():
    sol = Solution()
    result = sol.longestSubstring("aaabb", 3)
    assert result == 3


def test_case_1():
    sol = Solution()
    result = sol.longestSubstring("ababbc", 2)
    assert result == 5



from math import sqrt


def gcd2(a, b):
    x, y, lx, ly = 0, 1, 1, 0
    while b != 0:
        q = a // b
        a, b = b, a % b
        x, y, lx, ly = lx - q * x, ly - q * y, x, y
    return (lx, ly)


def inv(n, p):
    return gcd2(n, p)[0] % p


def solve(k):
    if k <= 2:
        return 1

    if not (k & 1):
        k //= 2

    if not (k % 2 and k % 5):
        return -1

    sq = int(sqrt(k))
    s = 1
    tbl = {}
    for i in range(sq + 2):
        if s not in tbl:
            tbl[s] = i
        s = s * 10 % k

    psinv = inv(pow(10, sq, k), k)
    m = inv(10, k)
    for i in range(sq + 2):
        if m in tbl:
            return sq * i + tbl[m] + 1
        m = m * psinv % k

    return -1


t = int(input())
for i in range(t):
    k = int(input())
    print(solve(k))



from buggy import Solution


def test_case_0():
    sol = Solution()
    result = sol.countPairs([3,1,2,2,2,1,3], 2)
    assert result == 4


def test_case_1():
    sol = Solution()
    result = sol.countPairs([1,2,3,4], 1)
    assert result == 0



from buggy import Solution


def test_case_0():
    sol = Solution()
    result = sol.arrayChange([1,2,4,6], [[1,3],[4,7],[6,1]])
    assert result == [3,2,7,1]


def test_case_1():
    sol = Solution()
    result = sol.arrayChange([1,2], [[1,3],[2,1],[3,2]])
    assert result == [2,1]



import bisect
import copy
import decimal
import fractions
import heapq
import itertools
import math
import random
import sys
import time
from collections import Counter, deque, defaultdict
from functools import lru_cache, reduce
from heapq import heappush, heappop, heapify, heappushpop, _heappop_max, _heapify_max


def _heappush_max(heap, item):
    heap.append(item)
    heapq._siftdown_max(heap, 0, len(heap) - 1)


def _heappushpop_max(heap, item):
    if heap and item < heap[0]:
        item, heap[0] = heap[0], item
        heapq._siftup_max(heap, 0)
    return item


from math import gcd as GCD

read = sys.stdin.read
readline = sys.stdin.readline
readlines = sys.stdin.readlines
write = sys.stdout.write
# import pypyjit
# pypyjit.set_param('max_unroll_recursion=-1')
# sys.set_int_max_str_digits(10**9)

N = int(readline())
D = list(map(int, readline().split()))
L0, C0, K0 = map(int, readline().split())
L1, C1, K1 = map(int, readline().split())
inf = 1 << 30
dp = [0] * (K0 + 1)
for d in D:
    prev = dp
    dp = [inf] * (K0 + 1)
    for k in range(K0 + 1):
        for kk in range(k, K0 + 1):
            dp[kk] = min(dp[kk], prev[k] + max(0, d - (kk - k) * L0 + L1 - 1) // L1)
ans = inf
for k0 in range(K0 + 1):
    k1 = dp[k0]
    if k1 <= K1:
        ans = min(ans, k0 * C0 + k1 * C1)
if ans == inf:
    ans = -1
print(ans)



import numpy as np
from buggy import add_bias


def test_bias_broadcast_shape() -> None:
    matrix = np.array([[1.0, 2.0, 3.0], [2.0, 4.0, 6.0]])
    bias = np.array([1.0, -1.0, 0.5])
    result = add_bias(matrix, bias)
    assert result.shape == (2, 3)



n, m = map(int, input().split())
s = [input() for i in range(n)]
a = 0
for i in range(1, (1 << n) - 1):
    c = [0] * 26
    for j in range(n):
        if (i >> j) & 1:
            for k in range(26):
                c[k] += chr(k + ord("a")) in s[j]
    a = max(a, c.count(m))
print(a)



N, M = map(int, input().split())
A = list(map(int, input().split()))
B = list(map(int, input().split()))
A.sort()
B.sort()
mx = max(A[-1], B[-1])


def is_ok(x):
    sell = len([a for a in A if a <= x])
    buy = len([b for b in B if b >= x])
    return sell >= buy


def binary_search(ng, ok):
    """
    初期値のng,okを受け取り,is_okを満たす最小(最大)のokを返す
    ng ok は  とり得る最小の値-1 とり得る最大の値+1
    最大最小が逆の場合はよしなにひっくり返す
    """
    while abs(ok - ng) > 1:
        mid = (ok + ng) // 2
        if is_ok(mid):
            ok = mid
        else:
            ng = mid
    return ok


print(binary_search(0, mx))



from buggy import Solution


def test_case_0():
    sol = Solution()
    result = sol.getMaximumConsecutive([1,3])
    assert result == 2


def test_case_1():
    sol = Solution()
    result = sol.getMaximumConsecutive([1,1,1,4])
    assert result == 8


def test_case_2():
    sol = Solution()
    result = sol.getMaximumConsecutive([1,4,10,3,1])
    assert result == 20



import math
import sys
import random
from collections import deque
from itertools import product

debug = lambda name, value, *args, **kwargs: print(
    f"{name}: {value}", *args, file=sys.stderr, **kwargs
)

mod = 1000000007
m = random.randint(1, 1000000)
iinf = 100000000000000005  # 1e18 + 5

input = lambda: sys.stdin.readline().strip()
# sys.stdin = open('input.txt', 'r')
# sys.stdout = open('output.txt', 'w')

dr = [1, -1, 0, 0]
dc = [0, 0, 1, -1]


def main():
    n, k = map(int, input().split())
    (*v,) = map(int, input().split())
    dp = [[iinf] * 2 for _ in range(k + 1)]
    dp[0][0] = 0
    for i in range(k):
        if i + 2 <= k:
            for j in range(2):
                dp[i + 2][j] = min(dp[i + 1][j], dp[i][j] + v[i + 1] - v[i])
        dp[i + 1][1] = min(dp[i + 1][1], dp[i][0])
    print(min(dp[k][0], dp[k][1]))


t = 1
# t = int(input())
for _ in range(t):
    main()



N, T = map(int, input().split())
A = list(map(int, input().split()))

horizontal = [[] for _ in range(N)]
vertical = [[] for _ in range(N)]
diagonal = [[], []]

ans = -1
for i, a in enumerate(A):
    row = (a - 1) // N
    col = (a - 1) % N
    horizontal[row].append(a)
    vertical[col].append(a)
    if row == col:
        diagonal[0].append(a)
    if row + col == N - 1:
        diagonal[1].append(a)
    if (
        (len(horizontal[row]) == N)
        or (len(vertical[col - 1]) == N)
        or (len(diagonal[0])) == N
        or (len(diagonal[1])) == N
    ):
        ans = i + 1
        break
print(ans)



n, a, b = map(int, input().split())
D = list(map(int, input().split()))

w = a + b
mod_D = []
for d in D:
    mod_D.append(d % w)

mod_D = sorted(list(set(mod_D)))
length = len(mod_D)
for i in range(length):
    mod_D.append(mod_D[i] + w)

for i in range(len(mod_D) - 1):
    if (mod_D[i + 1] - mod_D[i]) % w > b:
        print("Yes")
        exit()
print("No")



N = int(input())
a = list(input())
a_cost = list(map(int, input().split())) + [0]
zo = [0 for _ in range(N)]
oz = [0 for _ in range(N)]
for an in range(N):
    if a[an] == str(0 + an % 2):
        zo[an] = zo[an - 1]
        oz[an] = oz[an - 1] + a_cost[an]
    elif a[an] == str((1 + an) % 2):
        zo[an] = zo[an - 1] + a_cost[an]
        oz[an] = oz[an - 1]
min_m = 10**24
x = zo[-1]
y = oz[-1]
for n in range(N):
    min_m = min(min_m, zo[n] - oz[n] + y, oz[n] - zo[n] + x)
print(min_m)
# print(zo,oz)



import subprocess
import sys


def test_condefects_0():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='3 5\n3 5 2\n8 1 3\n1 2 2 3\n1 3 1 3\n1 1 1 1\n2 2 2 2\n3 3 1 1\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '2\n1\n11\n6\n10\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_1():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='1 1\n9\n100\n1 1 1 1\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '109\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )



s = str(input())
if s == "tourist":
    print(3858)
if s == "ksun48":
    print(3679)
if s == "Beng":
    print(3658)
if s == "Um_nik":
    print(3648)
if s == "apiad":
    print(3638)
if s == "Stonefeang":
    print(3630)
if s == "ecnerwala":
    print(3613)
if s == "mnbvmar":
    print(3555)
if s == "newbiedmy":
    print(3516)
if s == "semiexp":
    print(3481)



from typing import List, Optional, Tuple, Dict, Set
from collections import defaultdict, Counter, deque
import heapq
import math
import bisect
import itertools
import functools
class Solution:
    def findMedianSortedArrays(self, nums1: List[int], nums2: List[int]) -> float
        p1 = 0
        p2 = 0
        new = []
        while p1 < len(nums1) and p2 < len(nums2):
            if nums1[p1] < nums2[p2]:
                new.append(nums1[p1])
                p1 = 1
            else:
                new.append(nums2[p2])
                p2 += 1
        while p1 < len(nums1):
            new.append(nums1[p1])
            p2 += 1
        while p2 < len(nums2):
            new.append(nums2[p2])
            p1 += 1

        if len(new) % 2 == 0:
            index = len(new) // 2
            median = (new[index] + new[index-1]) / 2
        else:
            median = float(new[len(new)//2])
        return median



import subprocess
import sys


def test_condefects_0():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='5\n3 1 4 1 5\n4\n1 5 1\n2 4 3\n1 5 2\n1 3 3\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '2\n0\n0\n1\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )



from collections import defaultdict, deque, Counter
from heapq import heappush, heappop, heapify
import math
import bisect
import random
from itertools import permutations, accumulate, combinations, product
import sys
import string
from bisect import bisect_left, bisect_right
from math import factorial, ceil, floor, cos, radians, pi, sin
from operator import mul
from functools import reduce
from operator import mul
from functools import lru_cache

mod = 10**9 + 7
sys.setrecursionlimit(2147483647)
INF = 10**13


def LI():
    return list(map(int, sys.stdin.buffer.readline().split()))


def I():
    return int(sys.stdin.buffer.readline())


def LS():
    return sys.stdin.buffer.readline().rstrip().decode("utf-8").split()


def S():
    return sys.stdin.buffer.readline().rstrip().decode("utf-8")


def IR(n):
    return [I() for i in range(n)]


def LIR(n):
    return [LI() for i in range(n)]


def SR(n):
    return [S() for i in range(n)]


def LSR(n):
    return [LS() for i in range(n)]


def SRL(n):
    return [list(S()) for i in range(n)]


def MSRL(n):
    return [[int(j) for j in list(S())] for i in range(n)]


@lru_cache(maxsize=None)
def factorial(n):
    if n == 0:
        return 1
    else:
        return (n * factorial(n - 1)) % mod


class UnionFind:
    def __init__(self, n):
        # 負  : 根であることを示す。絶対値はランクを示す
        # 非負: 根でないことを示す。値は親を示す
        self.table = [-1] * n
        self.size = [1] * n
        self.group_num = n

    def root(self, x):
        if self.table[x] < 0:
            return x
        else:
            self.table[x] = self.root(self.table[x])
            return self.table[x]

    def get_size(self, x):
        r = self.root(x)
        return self.size[r]

    def is_same(self, x, y):
        return self.root(x) == self.root(y)

    def union(self, x, y):
        r1 = self.root(x)
        r2 = self.root(y)
        if r1 == r2:
            return
        # ランクの取得
        d1 = self.table[r1]
        d2 = self.table[r2]
        if d1 <= d2:
            self.table[r2] = r1
            self.size[r1] += self.size[r2]
            if d1 == d2:
                self.table[r1] -= 1
        else:
            self.table[r1] = r2
            self.size[r2] += self.size[r1]
        self.group_num -= 1


n = I()
P = LI()

fac = [1] * (n + 1)
inv = [1] * (n + 1)
for j in range(1, n + 1):
    fac[j] = fac[j - 1] * j % mod


inv[n] = pow(fac[n], mod - 2, mod)
for j in range(n - 1, -1, -1):
    inv[j] = inv[j + 1] * (j + 1) % mod


def comb(n, r):
    if r > n or n < 0 or r < 0:
        return 0
    return fac[n] * inv[n - r] * inv[r] % mod


U = UnionFind(n)
for i in range(n):
    if P[i] != -1:
        U.union(P[i] - 1, i)

k = 0
L = []
for j in range(n):
    if P[j] == -1:
        L += [U.get_size(j)]
        k += 1

dp = [0] * (k + 1)
dp[0] = 1
dp_new = dp[:]
for i in range(1, k + 1):
    for j in range(i):
        dp_new[j + 1] += dp[j] * L[i - 1]
    dp = dp_new[:]


if k:
    dp[1] = sum(L) - len(L)

cycle_count = 0
for i in range(1, k + 1):
    # i個のグループからなるサイクルがひとつできる
    buf = dp[i] * factorial(i - 1)
    buf %= mod
    buf *= pow(n, k - i, mod)
    buf %= mod
    cycle_count += buf


# すでに固定サイクルがあれば数える
cycle_count += (U.group_num - k) * pow(n, k, mod)
ans = n * pow(n, k, mod)
ans %= mod
ans = ans - cycle_count
ans %= mod


print((cycle_count + pow(n, k - 1, mod) * k) % mod)



n, m, t = map(int, input().split())
a = list(map(int, input().split()))
v = [0] * n
for i in range(m):
    x, y = map(int, input().split())
    v[x - 1] = y
for i in range(n - 1):
    t += v[i]
    if t > i:
        t -= a[i]
    else:
        print("No")
        break
else:
    print("Yes")



import sys

input = sys.stdin.readline
inf = 10**18


def read(dtype=int):
    return list(map(dtype, input().split()))


(n,) = read()
(s,) = read(str)
s = list(map(int, s))
c = read()
dp = [[inf, inf], [inf, inf]]
dp[s[0] ^ 1][0] = c[0]
dp[s[0]][0] = 0
for i in range(1, n):
    ndp = [[inf, inf], [inf, inf]]
    for k in range(2):
        cost = k and c[i]
        for x in range(2):
            for y in range(2):
                same = s[i] == x ^ k
                if y and same:
                    continue

                ndp[s[i] ^ k][same | y] = min(ndp[s[i] ^ k][same | y], dp[x][y] + cost)
    dp = ndp
print(min(map(min, dp)))



from buggy import Solution


def test_case_0():
    sol = Solution()
    result = sol.countGoodSubstrings("xyzzaz")
    assert result == 1


def test_case_1():
    sol = Solution()
    result = sol.countGoodSubstrings("aababcabc")
    assert result == 4



H, W = map(int, input().split())
A = [[int(i) for i in input().split()] for j in range(H)]
ans = [["." for h in range(W)] for w in range(H)]

for i in range(H):
    for j in range(W):
        if A[i][j] == 0:
            continue
        ans[i][j] = chr(64 + A[i][j])

for i in range(H):
    for j in range(W):
        if W - 1 == j:
            print(ans[i][j])
        else:
            print(ans[i][j], end=" ")



n, m = map(int, input().split())
a = list(map(int, input().split()))
b = list(map(int, input().split()))

c = [(i, 0) for i in a] + [(i, 1) for i in b]
c.sort()

ans = "No"
for i in range(n + m - 1):
    if c[i][1] == c[i + 1][1]:
        ans = "Yes"

print(ans)



N, mod = map(int, input().split())

con = [[0] * N for _ in range(N)]
sep = [[0] * (N + 1) for _ in range(N)]
con[0][0] = 1
sep[0][1] = 1
for n in range(N)[1:]:
    con[n][0] = 1
    for m in range(n + 1)[1:]:
        con[n][m] = con[n - 1][m] + con[n - 1][m - 1] * 3 + sep[n - 1][m]
        con[n][m] %= mod
    for m in range(n + 2)[2:]:
        sep[n][m] = con[n - 1][m - 2] * 2 + sep[n - 1][m - 1]
        sep[n][m] %= mod

print(*con[-1])



import numpy as np
from buggy import normalize_rows


def test_normalize_rows_nonsquare() -> None:
    m = np.array([[1.0, 0.0, 0.0], [0.0, 3.0, 4.0]])
    result = normalize_rows(m)
    assert result.shape == (2, 3)
    np.testing.assert_allclose(result[0], [1.0, 0.0, 0.0])
    np.testing.assert_allclose(result[1], [0.0, 0.6, 0.8])


def test_normalize_rows_square() -> None:
    """Square matrix gives wrong values -- the bug is subtle here."""
    m = np.array([[3.0, 4.0], [0.0, 0.0]])
    result = normalize_rows(m)
    assert result.shape == (2, 2)
    np.testing.assert_allclose(np.linalg.norm(result[0]), 1.0)
    np.testing.assert_array_equal(result[1], [0.0, 0.0])


def test_normalize_rows_single_row() -> None:
    m = np.array([[5.0, 0.0]])
    result = normalize_rows(m)
    assert result.shape == (1, 2)
    np.testing.assert_allclose(result[0], [1.0, 0.0])



import sys

input = sys.stdin.readline

N, H = map(int, input().split())
X = list(map(int, input().split()))
X = [0] + X
P = [0 for i in range(N + 1)]
F = [0 for i in range(N + 1)]
for i in range(1, N):
    P[i], F[i] = map(int, input().split())

inf = 10**8
dp = [[[inf for i in range(H + 1)] for i in range(H + 1)] for i in range(N + 1)]
for i in range(H + 1):
    dp[0][H][i] = 0

for n in range(N - 1):
    d = X[n + 1] - X[n]
    for h in range(d, H + 1):
        for hh in range(H - d + 1):
            if dp[n][h][hh] == inf:
                continue
            # 使わない
            dp[n + 1][h - d][hh + d] = min(dp[n + 1][h - d][hh + d], dp[n][h][hh])
            # 往路で使う
            hhh = min(H, h - d + F[n + 1])
            dp[n + 1][hhh][hh + d] = min(dp[n + 1][hhh][hh + d], dp[n][h][hh] + P[n + 1])
            # 復路で使う
            hhh = hh + d
            if hhh == H:
                for k in range(hhh - F[n + 1], hhh + 1):
                    dp[n + 1][h - d][k] = min(dp[n + 1][hhh][k], dp[n][h][hh] + P[n + 1])
            elif hhh >= F[n + 1]:
                dp[n + 1][h - d][hhh - F[n + 1]] = min(
                    dp[n + 1][h - d][hhh - F[n + 1]], dp[n][h][hh] + P[n + 1]
                )

d = X[N] - X[N - 1]
ans = inf
for h in range(d, H - d + 1):
    ans = min(ans, dp[N - 1][h + d][h - d])
if ans == inf:
    ans = -1
print(ans)



N, M = map(int, input().split())
A = list(map(int, input().split()))

dp = [[0] * N for i in range(M)]
dp[0][0] = A[0]
for i in range(1, N):
    dp[0][i] = max(A[i], dp[0][i - 1])

for i in range(1, M):
    for j in range(i, N):
        dp[i][j] = max(dp[i - 1][j - 1] + A[j] * (i + 1), dp[i][j - 1])

print(dp[-1][-1])



#!/usr/bin/env python
import os
import sys
from io import BytesIO, IOBase


def main():
    n, m = map(int, input().split())
    x = list(map(int, input().split()))
    d = list(map(int, input().split()))
    arrow = [None] * (10**6 + 1)
    currentMin = 1
    currentMinLoc = 1
    direction = 1
    length = 10**6
    zeroDict = dict()
    for index, elem in enumerate(d):
        if length == 0:
            break
        prev_pivot = currentMinLoc - currentMin * direction
        current_pivot = prev_pivot + direction * elem
        currentEnd = currentMinLoc + (length - 1) * direction
        if (currentMinLoc - current_pivot) * direction > 0:
            currentMin = abs(currentMinLoc - current_pivot)
        elif (current_pivot - currentEnd) * direction > 0:
            direction = -direction
            currentMinLoc = currentEnd
            currentMin = abs(currentEnd - current_pivot)
        else:
            if abs(current_pivot - currentMinLoc) <= abs(currentEnd - current_pivot):
                for j in range(currentMinLoc, current_pivot + direction, direction):
                    arrow[j] = current_pivot * 2 - j
                    if arrow[j] == j:
                        zeroDict[j] = index + 1
                    length -= 1
                currentMin = 1
                currentMinLoc = current_pivot + direction
            else:
                for j in range(current_pivot, currentEnd + direction, direction):
                    arrow[j] = current_pivot * 2 - j
                    if arrow[j] == j:
                        zeroDict[j] = index + 1
                    length -= 1
                currentMin = 1
                direction = -direction
                currentMinLoc = current_pivot + direction

    graph = [[] for _ in range(10**6 + 1)]
    for i in range(10**6 + 1):
        if arrow[i] is not None and arrow[i] != i:
            graph[arrow[i]].append(i)
    num = [0]
    for i in range(1, 10**6 + 1):
        if arrow[i] is None:
            num.append(abs(i - currentMinLoc) + currentMin)
        else:
            num.append(None)
    dist = [None] * (10**6 + 1)
    for i in range(10**6 + 1):
        if arrow[i] is None:
            stack = [i]
            dist[i] = 0
            while stack:
                v = stack.pop()
                for u in graph[v]:
                    if dist[u] is None:
                        dist[u] = dist[v] + 1
                        num[u] = num[v]
                        stack.append(u)
        elif arrow[i] == i:
            stack = [i]
            dist[i] = zeroDict[i]
            num[i] = 0
            while stack:
                v = stack.pop()
                for u in graph[v]:
                    if dist[u] is None:
                        dist[u] = dist[v]
                        num[u] = num[v]
                        stack.append(u)
    for elem in x:
        if num[elem] == 0:
            print("Yes " + str(dist[elem]))
        else:
            print("No " + str(num[elem] if (dist[elem] + direction) % 2 == 0 else -num[elem]))


# region fastio

BUFSIZE = 8192


class FastIO(IOBase):
    newlines = 0

    def __init__(self, file):
        self._file = file
        self._fd = file.fileno()
        self.buffer = BytesIO()
        self.writable = "x" in file.mode or "r" not in file.mode
        self.write = self.buffer.write if self.writable else None

    def read(self):
        while True:
            b = os.read(self._fd, max(os.fstat(self._fd).st_size, BUFSIZE))
            if not b:
                break
            ptr = self.buffer.tell()
            self.buffer.seek(0, 2), self.buffer.write(b), self.buffer.seek(ptr)
        self.newlines = 0
        return self.buffer.read()

    def readline(self):
        while self.newlines == 0:
            b = os.read(self._fd, max(os.fstat(self._fd).st_size, BUFSIZE))
            self.newlines = b.count(b"\n") + (not b)
            ptr = self.buffer.tell()
            self.buffer.seek(0, 2), self.buffer.write(b), self.buffer.seek(ptr)
        self.newlines -= 1
        return self.buffer.readline()

    def flush(self):
        if self.writable:
            os.write(self._fd, self.buffer.getvalue())
            self.buffer.truncate(0), self.buffer.seek(0)


class IOWrapper(IOBase):
    def __init__(self, file):
        self.buffer = FastIO(file)
        self.flush = self.buffer.flush
        self.writable = self.buffer.writable
        self.write = lambda s: self.buffer.write(s.encode("ascii"))
        self.read = lambda: self.buffer.read().decode("ascii")
        self.readline = lambda: self.buffer.readline().decode("ascii")


sys.stdin, sys.stdout = IOWrapper(sys.stdin), IOWrapper(sys.stdout)
input = lambda: sys.stdin.readline().rstrip("\r\n")

# endregion

if __name__ == "__main__":
    main()



from buggy import Solution


def test_case_0():
    sol = Solution()
    result = sol.maxNiceDivisors(5)
    assert result == 6


def test_case_1():
    sol = Solution()
    result = sol.maxNiceDivisors(8)
    assert result == 18



n, m = map(int, input().split())
c = input().split()
d = input().split()
p = list(map(int, input().split()))
ds = {d[i]: p[1 + i] for i in range(m)}
s = 0
for i in c:
    if i not in ds:
        print(p[0])
    else:
        s += ds[i]
print(s)



def wrap_index(index: int, length: int) -> int:
    """Convert a possibly-negative index to a valid position in a circular buffer.

    Examples: wrap_index(-1, 5) -> 4, wrap_index(7, 5) -> 2
    """
    # Bug: uses int() truncation toward zero instead of Python's floor modulo.
    # For negative numbers, int(-1/5) = 0, so this returns -1 + 0*5 = -1.
    return index - int(index / length) * length


def circular_get(buf: list[int], index: int) -> int:
    """Get element at circular index from buffer."""
    pos = wrap_index(index, len(buf))
    return buf[pos]



from typing import List, Optional, Tuple, Dict, Set
from collections import defaultdict, Counter, deque
import heapq
import math
import bisect
import itertools
import functools


class Solution:
    def minDistance(self, word1: str, word2: str) -> int:
        m = len(word1)
        n = len(word2)
        dp = []
        for i in range(m + 1):
            dp.append([0] * (n + 1))
        for i in range(m + 1):
            dp[i][0] = i
        for i in range(n + 1):
            dp[0][i] = i
        for i in range(1, m + 1):
            for j in range(1, n + 1):
                if word1[i] == word2[j]:
                    dp[i][j] = dp[i - 1][j - 1]
                else:
                    dp[i][j] = min(dp[i][j - 1], dp[i - 1][j]) + 1
        return dp[-1][-1]



import subprocess
import sys


def test_condefects_0():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='6\n3 ABC ABC\n1 C B\n1 B C\n2 AB BA\n2 BA AB\n3 CCB ABA\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = 'Yes\nYes\nNo\nYes\nNo\nYes\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_1():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='7\n5 ABABA BABAB\n5 ABCBC BBABA\n5 CCCCC CBABC\n5 BBAAA AAABB\n5 AAABB BBAAA\n5 ACACB BAACB\n5 ACACB BBACA\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = 'No\nYes\nYes\nNo\nYes\nYes\nNo\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )



from buggy import get_all


def test_gather_returns_errors_with_results() -> None:
    urls = ["https://good.com", "https://bad.com", "https://also-good.com"]
    results = get_all(urls)
    assert len(results) == 3
    assert results[0] == "data from https://good.com"
    assert isinstance(results[1], ValueError)
    assert results[2] == "data from https://also-good.com"



from typing import List, Optional, Tuple, Dict, Set
from collections import defaultdict, Counter, deque
import heapq
import math
import bisect
import itertools
import functools


class Solution:
    # odd/even diameter couned individually
    def countSubgraphsForEachDiameter(self, n: int, edges: List[List[int]]) -> List[int]:
        adj = [[] for i in range(n)]
        for u, v in edges:
            adj[u - 1].append(v - 1)
            adj[v - 1].append(u - 1)

        def comb(p, q):  # merge p and q, res[max(i,j)]+=p[i]*q[j]
            if len(q) < len(p):
                p, q = q, p
            res = [0] * len(q)
            res[0] = p[0] * q[0]
            for i in range(1, len(p)):
                p[i] += p[i - 1]
            for i in range(1, len(q)):
                q[i] += q[i - 1]
            for i in range(1, len(p)):
                res[i] = p[i] * q[i] - p[i - 1] * q[i - 1]
            for i in range(len(p), len(q)):
                res[i] = (q[i] - q[i - 1]) * p[-1]
            return res

        def dfs(r, p):  # num of subtree rooted at r with given depth
            d = [1]
            for v in undefined_list:
                if v == p:
                    continue
                t = [1] + dfs(v, r)
                d = comb(t, d)
            return d

        # end dfs
        ans = [0] * n
        # odd diameter with (u,v) as center edge
        for u, v in edges:
            u -= 1
            v -= 1
            p = dfs(u, v)
            q = dfs(v, u)
            for i in range(min(len(p), len(q))):
                ans[i + i + 1] += p[i] * q[i]
        # even diamter with v as center vertex
        for v in range(n):
            if len(adj[v]) == 1:
                continue
            tree = [1] + dfs(adj[v][0], v)  # tree with depth
            curr = [0] * n
            for u in adj[v][1:]:
                q = [1] + dfs(u, v)
                # curr tree + new small
                j = 1
                t = q[1] + 1  # prefix sum of q
                for i in range(4, n, 2):
                    while j + 1 < min(i // 2, len(q)):
                        j += 1
                        t += q[j]
                    curr[i] *= t
                # curr tree + same height
                for i in range(min(len(tree), len(q))):
                    curr[i + i] += tree[i] * q[i]
                tree = comb(tree, q)
            for i in range(2, n, 2):
                ans[i] += curr[i]
        # end
        return ans[1:]



from buggy import run


def test_cancellation_propagates() -> None:
    status, log = run()
    assert status == "cancelled", f"expected cancellation, got {status!r}"
    assert "suppressed" not in log, "CancelledError was suppressed instead of propagating"



N, M, P = map(int, input().split())
A = list(map(int, input().split()))
B = list(map(int, input().split()))
B.sort()
S = [B[0]]
for j in range(1, M):
    S.append(B[j] + S[j - 1])

res = 0
for i in range(N):
    l, r = -1, M
    while l + 1 < r:
        m = (l + r) // 2
        if A[i] + B[m] <= P:
            l = m
        else:
            r = m
    res += (S[l] + A[i] if l >= 0 else 0) + P * (M - r)
print(res)



import asyncio

results: list[int] = []


async def background_work(value: int) -> None:
    await asyncio.sleep(0)
    results.append(value * 10)


async def schedule_jobs(values: list[int]) -> list[int]:
    results.clear()
    for v in values:
        asyncio.create_task(background_work(v))
    return list(results)


def run_jobs(values: list[int]) -> list[int]:
    return asyncio.run(schedule_jobs(values))



import heapq

N, M = map(int, input().split())
edge = [[] for i in range(N)]
for i in range(M):
    l, d, k, c, A, B = map(int, input().split())
    A -= 1
    B -= 1
    edge[B].append((l, d, k, c, A))
P = [(-(10**20), N - 1)]
heapq.heapify(P)
ans = [-1] * N
while P:
    time, pos = heapq.heappop(P)
    time *= -1
    if ans[pos] > 0:
        continue
    ans[pos] = time
    for l, d, k, c, A in edge[pos]:
        if ans[A] > 0:
            continue
        kt = (time - c - l) // d
        if kt < 0:
            continue
        if kt > k - 1:
            kt = k - 1
        heapq.heappush(P, (-kt * d - l, A))
for a in ans:
    print(a if a >= 0 else "Unreachable")



# INF = 1<<60 # 1152921504606846976 ~ 10^18

# def meguru_bisect(ng, ok):
#     while ok - ng > 1:
#         mid = (ng + ok) // 2
#         if is_ok(mid):  # 小さくできる
#             ok = mid
#         else:
#             ng = mid  # 大きくしないといけない
#     return ok

import sys


def input():
    return sys.stdin.readline()[:-1]


# sys.setrecursionlimit(10**6)

N, Q = map(int, input().split())
A = list(map(int, input().split()))

types = []
from collections import defaultdict

d = defaultdict(int)
for a in A:
    if d[a] % 2 == 0:
        types.append(0)
    else:
        types.append(1)
    d[a] += 1

cumsum = [0] * (Q + 1)
l = 0
for i, t in enumerate(types):
    if t == 0:
        l += 1
    else:
        l -= 1
    if i < Q + 2:
        cumsum[i + 1] = cumsum[i] + l

ans = [0] * (N + 1)
pre = [-1] * (N + 1)
for i, a in enumerate(A):
    if pre[a] == -1:
        pre[a] = i + 1
    else:
        ans[a] += cumsum[i] - cumsum[pre[a] - 1]
        pre[a] = -1

for i, x in enumerate(pre):
    if x != -1:
        ans[i] += cumsum[-1] - cumsum[pre[i] - 1]
print(f"{ans[1:]}")



from buggy import Solution


def test_case_0():
    sol = Solution()
    result = sol.findAllRecipes(["bread"], [["yeast","flour"]], ["yeast","flour","corn"])
    assert result == ["bread"]


def test_case_1():
    sol = Solution()
    result = sol.findAllRecipes(["bread","sandwich"], [["yeast","flour"],["bread","meat"]], ["yeast","flour","meat"])
    assert result == ["bread","sandwich"]


def test_case_2():
    sol = Solution()
    result = sol.findAllRecipes(["bread","sandwich","burger"], [["yeast","flour"],["bread","meat"],["sandwich","meat","bread"]], ["yeast","flour","meat"])
    assert result == ["bread","sandwich","burger"]



H, W, N, h, w = map(int, input().split())
A = [list(map(int, input().split())) for _ in range(H)]
ans = [[0] * (W - w + 1) for _ in range(H - h + 1)]
dic = {}
for i in range(H):
    for j in range(W):
        if i >= h or j >= w:
            a = A[i][j]
            if a in dic:
                dic[a] += 1
            else:
                dic[a] = 1
ans[0][0] = len(dic)
for i in range(H - h + 1):
    dic2 = dic.copy()
    for j in range(W - w):
        for k in range(h):
            a = A[i + k][j + w]
            dic2[a] -= 1
            if dic2[a] == 0:
                dic2.pop(a)
            a = A[i + k][j]
            if a in dic2:
                dic2[a] += 1
            else:
                dic2[a] = 1
        ans[i][j + 1] = len(dic2)
    if i < H - h:
        for k in range(1, w):
            a = A[i + h][k]
            dic[a] -= 1
            if dic[a] == 0:
                dic.pop(a)
            a = A[i][k]
            if a in dic:
                dic[a] += 1
            else:
                dic[a] = 1
        ans[i + 1][0] = len(dic)
for i in range(H - h + 1):
    print(*ans[i])



n = int(input())
adict = dict(zip(list(range(1, n + 1)), list(map(int, input().split()))))
iset = set(adict.keys())
aset = set(adict.values())
last = (iset - aset).pop()
acc = []
while last != -1:
    acc.append(last)
    last = adict[last]
acc.reverse()
print(acc)



n = int(input())
a = list(map(int, input().split()))
t = []
for i in a:
    if len(t) == 0 or t[-1][0] != i:
        t.append([i, 1])
    else:
        t[-1][1] += 1
mx0 = 0
mx1 = 0
c0 = 0
c1 = 1
ind0 = 0
ind1 = 0
for i in range(n):
    c0 = max(c0, 0)
    c1 = max(c1, 0)
    if a[i] == 0:
        c0 += 1
        c1 -= 1
    else:
        c1 += 1
        c0 -= 1
    mx0 = max(mx0, c0)
    mx1 = max(mx1, c1)
print(mx0 + mx1 + 1)



from buggy import Solution


def test_case_0():
    sol = Solution()
    result = sol.kthFactor(12, 3)
    assert result == 3


def test_case_1():
    sol = Solution()
    result = sol.kthFactor(7, 2)
    assert result == 7


def test_case_2():
    sol = Solution()
    result = sol.kthFactor(4, 4)
    assert result == -1



import subprocess
import sys


def test_condefects_0():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='4 2\n1 2 3 4\n4 3 1 2\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '2\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_1():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='3 3\n1 2 3\n3 1 2\n1 2 3\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '0\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_2():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='10 10\n4 10 7 2 8 3 9 1 6 5\n3 6 2 9 1 8 10 7 4 5\n9 3 4 5 7 10 1 8 2 6\n7 3 1 8 4 9 5 6 2 10\n5 2 1 4 10 7 9 8 3 6\n5 8 1 6 9 3 2 4 7 10\n8 10 3 4 5 7 2 9 6 1\n3 10 2 7 8 5 1 4 9 6\n10 6 1 5 4 2 3 8 9 7\n4 5 9 1 8 2 7 6 3 10\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '6\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_3():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='2 1\n1 2\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '0\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_4():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='2 1\n2 1\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '0\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )



S = input()


def inverse(n, d):
    return n * pow(d, -1, MOD) % MOD


MOD = 998244353
comb = [[0] * len(S) for _ in range(len(S))]
for i in range(len(S)):
    for j in range((i + 1) // 2 + 1):
        if j == 0:
            comb[i][j] = i + 1
        else:
            comb[i][j] = comb[i][j - 1]
            comb[i][j] *= inverse(i + 1 - j, j + 1)
            comb[i][j] %= MOD

cnt = [0] * 26
for i in range(len(S)):
    cnt[ord(S[i]) - ord("a")] += 1

dp = [[0] * (len(S) + 1) for _ in range(27)]
dp[0][0] = 1
SUM = cnt[0]
for i in range(1, 27):
    for j in range(SUM + 1):
        dp[i][j] += dp[i - 1][j]
        dp[i][j] %= MOD
        for k in range(1, min(j + 1, cnt[i - 1] + 1)):
            J, K = j, k
            if J == K:
                dp[i][j] += dp[i - 1][j - k]
                dp[i][j] %= MOD
            else:
                if j % 2 == 0 and K > j // 2:
                    K -= (K - j // 2) * 2
                elif j & 2 == 1 and K > j // 2:
                    K -= (K - j // 2 - 1) * 2 + 1
                dp[i][j] += comb[J - 1][K - 1] * dp[i - 1][j - k]
                dp[i][j] %= MOD
    if i < 26:
        SUM += cnt[i]

ans = 0
for i in range(1, len(S) + 1):
    ans += dp[-1][i]
    ans %= MOD

print(ans)



N, M = map(int, input().split())
A = list(map(int, input().split()))
B = list(map(int, input().split()))

ans = "No"
C = sorted(A + B)
for i in range(len(C) - 1):
    if (C[i] in A and C[i + 1] in A) or (C[i] in B and C[i + 1] in B):
        ans = "Yes"
print(ans)



import subprocess
import sys


def test_condefects_0():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='3 2\nred green blue\nblue red\n800 1600 2800\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '5200\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_1():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='3 2\ncode queen atcoder\nking queen\n10 1 1\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '21\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_2():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='1 1\na\nb\n1 1000\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '1\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_3():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='22 75\nsjrggtycgcqfpo mrouhjkntqryhji x whfut mjcdlivnuxmrdf vwulbdkc ncksizgzajkhyjujlks yluacvrayggw ekfdsqcfvmqbocbomj lcgs tft l ks ycebeurgvjyospdhvuyf ubspxsiuytqrwmv yfezuja arenexc ueahebyvsak agmaygzyfnzpqftobt gpuamk yluacvrayggw mopodotkrjzri\nagmaygzyfnzpqftobt ak amahn arenexc bbo cdqciuehvfndghrxxnm cyurw dcc dpjimsfbuslks ehxmoy ekfdsqcfvmqbocbomj eycanjlbe fazdprco fbagzrxkprtgbo fdrsqfseqsdfcgdenmrf fnd fsvzlotlfw gpuamk hjphvs hk iblsrjuyfskndapwjef jdcqcrxvyaherm jtepxhnve kfxiwhfzfksnawg ks l lbdnfhfxamo lcgs ll mliygnfddeswmxuvh mopodotkrjzri ncksizgzajkhyjujlks nviaztpcmxlvova ofklitxaoykfoqk otuayxmwv ozjaraefml pdzfjmcjcmsaaskex pnzijdifyervjaoalc ppgykichjt pphykinkezplvfja qjhwkcexecmdk qqtqbxjjmqn qrjolv qv rahdigyoj rnhrhctntzly rqsfskevfxphcqoq s ssjoatryxmbwxbwex szafzjmdtsbyldhylcg sznjyarkuprerivhub tft tgqngguftuahalwvjw tvnr ubelzv ubspxsiuytqrwmv ueahebyvsak ujidhukzwzc ukfnlxfcevkjezqs um veergcgmonctcsvypol vwulbdkc wkrpiuiasfkjexpftzn woeyom wseqzd xovldqfza ycebeurgvjyospdhvuyf yf yfezuja yfkilrixodiwqcyre yluacvrayggw yngvmhneqyjdu zakbzrkjwqdyuxdvo zg zljxioyundvpndabsb\n3889 2778 7931 6678 5919 8952 10000 49 3547 1261 7434 951 7318 8430 7391 3993 7823 4503 1963 2346 4972 2406 3854 661 3031 989 5851 8228 5263 1412 7499 9178 819 2436 3672 571 4795 5549 1554 3967 9215 289 2925 6019 6141 2583 3969 119 5015 281 1993 7200 8835 518 3781 6616 5865 9920 8504 3545 7881 783 8490 214 2082 2285 1943 307 1164 9976 3933 3047 6095 3812 6978 8593\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '102643\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_4():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='14 22\nf nqwid zqiygwf efiwpx umdwugbfgno b pafgkirrueoezabcql cdwmklvyzvuughetw dnwhrssyiybitcoowlhm lbzlnvmosmvqub nrqa hacqfjcmwupoxonavvx rdpgtrdwalvffknwibh svprpymskznab\naqhsxzguseuexlycgub b bqgcrgvwalcptutzxs bxrmqdgqjsosencrbwk ddflgtixbnagkqrzes dnwhrssyiybitcoowlhm hacqfjcmwupoxonavvx krihwqplf lbzlnvmosmvqub lkchithcdezsaef llvucvubyozfmttxtlsr ltvmmzmslqdxqiifz npjgqftvwgukjwsekfcg nrqa quwssucxrmywerf rdpgtrdwalvffknwibh rufxdsqchjkuxpxng shaszrenhlbz spkq svprpymskznab ws ziqdziycnynthgmdav\n823 2805 732 9390 8151 8826 10000 4576 7174 4485 5046 7157 4375 4223 6693 1696 1083 8099 9938 8968 2778 8723 4141\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '36108\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )



def main():
    N, M = map(int, input().split())
    A = list(map(int, input().split()))
    A.sort()
    B = []

    for n in range(N - 1):
        B.append(A[n + 1] - A[n])

    kyori = 0
    kazu = 1
    i = 0
    ans = 0
    for b in B:
        kyori += b
        kazu += 1
        if kyori >= M:
            while kyori >= M:
                kyori -= B[i]
                i += 1
                kazu -= 1
        ans = max(ans, kazu)
    print(ans)


if __name__ == "__main__":
    main()



n, t, l = map(int, input().split())
mat = [[t + 1] * n for i in range(n)]
for i in range(1, t + 1):
    u, v = map(int, input().split())
    mat[u - 1][v - 1] = i


def mul(A, B):
    res = [[t + 1] * n for i in range(n)]
    for i in range(n):
        for j in range(n):
            for k in range(n):
                res[i][k] = min(res[i][j], max(A[i][j], B[j][k]))
    return res


def mat_pow(mat, k):
    res = [[t + 1] * n for i in range(n)]
    for i in range(n):
        res[i][i] = 0
    while k:
        if k & 1:
            res = mul(res, mat)
        mat = mul(mat, mat)
        k >>= 1
    return res


res = mat_pow(mat, l)
res = res[0]
for i in range(n):
    if res[i] > t:
        res[i] = -1
print(*res)



"""llmdebug MCP server — expose crash snapshots and RCA workflows to IDE agents.

Provides tools for programmatic access to debug snapshots and RCA state:
- llmdebug_diagnose: Concise crash summary optimized for LLM consumption
- llmdebug_show: Full expanded JSON snapshot
- llmdebug_list: List available snapshots
- llmdebug_frame: Detailed view of a specific stack frame
- llmdebug_git_context: On-demand enhanced git metadata for crash triage
- llmdebug_diff: Compare two snapshots with structured delta
- llmdebug_hypothesize: Generate ranked root-cause hypotheses
- llmdebug_rca_status / llmdebug_rca_history: RCA state visibility

Run with: llmdebug-mcp (stdio transport for IDE integration)
"""

from __future__ import annotations

import json
import logging
import os
import sys
from typing import Any

# Guard import — mcp is optional
try:
    from mcp.server.fastmcp import FastMCP
except ImportError:
    print(
        "llmdebug MCP server requires: pip install llmdebug[mcp]",
        file=sys.stderr,
    )
    raise SystemExit(1) from None

from ._debug_session import coerce_crash_frame_index
from ._formatting import (
    build_source_text,
    format_enhanced_git_context_text,
    format_local_value,
)
from ._snapshot_loader import (
    find_all_snapshots,
    load_snapshot,
    load_snapshot_raw,
    resolve_snapshot_ref,
)
from .git_context import get_enhanced_git_context
from .rca_state import (
    advance_rca_attempt,
    build_failure_signature,
    derive_session_id,
    get_rca_history,
    get_rca_status,
)
from .snapshot_diff import diff_snapshots, format_diff_text, structured_diff_summary

logger = logging.getLogger("llmdebug.mcp")

mcp = FastMCP("llmdebug")


# ---------------------------------------------------------------------------
# Text formatting helpers (plain-text, no Rich dependency)
# ---------------------------------------------------------------------------


def _format_local_value_str(key: str, value: Any, meta: dict[str, Any] | None) -> str:
    """Format a single local variable as a one-line string for MCP text output."""
    name, type_str, value_str = format_local_value(key, value, meta)
    type_part = f"  ({type_str})" if type_str else ""
    return f"  {name} = {value_str}{type_part}"


def _format_frame_text(frame: dict[str, Any], index: int, is_crash: bool) -> str:
    """Format a single stack frame as plain text."""
    file_rel = frame.get("file_rel", frame.get("file", "?"))
    line = frame.get("line", "?")
    func = frame.get("function", "?")
    prefix = "[CRASH] " if is_crash else ""

    parts: list[str] = [f"{prefix}Frame #{index}: {file_rel}:{line} in {func}"]

    # Source
    source = frame.get("source", {})
    if source:
        parts.append("")
        parts.append("Source:")
        parts.append(build_source_text(source, frame.get("line", 0)))

    # Locals
    locals_dict = frame.get("locals", {})
    locals_meta = frame.get("locals_meta", {})
    if isinstance(locals_dict, dict) and locals_dict:
        parts.append("")
        parts.append("Locals:")
        for k, v in locals_dict.items():
            if k.startswith("@py_"):
                continue
            parts.append(_format_local_value_str(k, v, locals_meta.get(k)))

    return "\n".join(parts)


# format_enhanced_git_context_text imported from ._formatting as format_enhanced_git_context_text


def _effective_gate_mode(gate_mode: str | None) -> str:
    candidate = (gate_mode or os.getenv("LLMDEBUG_RCA_GATE_MODE", "soft")).strip().lower()
    if candidate in {"off", "soft", "strict"}:
        return candidate
    return "soft"


def _extract_evidence_refs(snapshot: dict[str, Any]) -> list[str]:
    refs: list[str] = []
    exc = snapshot.get("exception", {})
    if isinstance(exc, dict):
        exc_type = str(exc.get("type") or "")
        exc_msg = str(exc.get("message") or "")
        if exc_type:
            refs.append(f"exception:{exc_type}")
        if exc_msg:
            refs.append(f"exception_message:{exc_msg[:80]}")

    frames = snapshot.get("frames", [])
    if isinstance(frames, list) and frames:
        crash_idx = coerce_crash_frame_index(snapshot)
        frame: dict[str, Any] | None = None
        if 0 <= crash_idx < len(frames) and isinstance(frames[crash_idx], dict):
            frame = frames[crash_idx]
        elif isinstance(frames[0], dict):
            frame = frames[0]
        if frame:
            file_rel = str(frame.get("file_rel", frame.get("file", "?")))
            line = int(frame.get("line") or 0)
            func = str(frame.get("function") or "?")
            refs.append(f"crash_frame:{file_rel}:{line}:{func}")

    pytest_block = snapshot.get("pytest", {})
    if isinstance(pytest_block, dict):
        nodeid = str(pytest_block.get("nodeid") or "")
        if nodeid:
            refs.append(f"pytest_nodeid:{nodeid}")
    return refs


def _advance_rca(
    *,
    out_dir: str,
    snapshot: dict[str, Any] | None,
    action: str,
    gate_mode: str | None,
    evidence_refs: list[str] | None = None,
    hypothesis: str | None = None,
    experiment_plan: str | None = None,
    proposed_change_summary: str | None = None,
    verification_result: dict[str, Any] | None = None,
    exploratory: bool = False,
    override_reason: str | None = None,
    fallback_stdout: str = "",
    fallback_stderr: str = "",
) -> dict[str, Any] | None:
    try:
        session_id = derive_session_id(snapshot, out_dir=out_dir, fallback="mcp")
        signature = build_failure_signature(
            snapshot,
            stdout=fallback_stdout,
            stderr=fallback_stderr,
        )
        return advance_rca_attempt(
            out_dir=out_dir,
            session_id=session_id,
            action=action,
            failure_signature=signature,
            evidence_refs=evidence_refs or [],
            hypothesis=hypothesis,
            experiment_plan=experiment_plan,
            proposed_change_summary=proposed_change_summary,
            verification_result=verification_result,
            gate_mode=_effective_gate_mode(gate_mode),
            exploratory=exploratory,
            override_reason=override_reason,
        )
    except Exception as exc:  # pragma: no cover - defensive path
        logger.warning("RCA state update failed: %s: %s", type(exc).__name__, exc)
        return None


def _advance_rca_with_snapshot_refs(
    *,
    out_dir: str,
    snapshot: dict[str, Any] | None,
    action: str,
    gate_mode: str | None,
    exploratory: bool = False,
    extra_evidence_refs: list[str] | None = None,
    hypothesis: str | None = None,
    experiment_plan: str | None = None,
    proposed_change_summary: str | None = None,
    verification_result: dict[str, Any] | None = None,
) -> dict[str, Any] | None:
    refs = _extract_evidence_refs(snapshot) if isinstance(snapshot, dict) else []
    if extra_evidence_refs:
        refs.extend(extra_evidence_refs)
    return _advance_rca(
        out_dir=out_dir,
        snapshot=snapshot,
        action=action,
        gate_mode=gate_mode,
        evidence_refs=refs,
        hypothesis=hypothesis,
        experiment_plan=experiment_plan,
        proposed_change_summary=proposed_change_summary,
        verification_result=verification_result,
        exploratory=exploratory,
    )


def _format_tool_response(
    body: str,
    *,
    rca_record: dict[str, Any] | None = None,
    structured_delta: dict[str, Any] | None = None,
) -> str:
    sections = [body]
    if structured_delta is not None:
        structured_text = json.dumps(
            structured_delta,
            indent=2,
            ensure_ascii=False,
        )
        sections.append("## Structured Diff\n" + structured_text)
    if rca_record is not None:
        rca_view = {
            "attempt_id": rca_record.get("attempt_id"),
            "session_id": rca_record.get("session_id"),
            "state": rca_record.get("state"),
            "failure_signature": rca_record.get("failure_signature"),
            "repeat_count": rca_record.get("repeat_count"),
            "allowed_actions": rca_record.get("allowed_actions", []),
            "next_required": rca_record.get("next_required", []),
            "gate_feedback": rca_record.get("gate_feedback", {}),
            "warnings": rca_record.get("gate_warnings", []),
        }
        sections.append("## RCA\n" + json.dumps(rca_view, indent=2, ensure_ascii=False))
        sections.append(
            "## RCA Prompt Contract\n"
            "1) Observed failure (cite evidence)\n"
            "2) Root-cause hypothesis (falsifiable)\n"
            "3) Why evidence supports it\n"
            "4) Minimal fix plan\n"
            "5) Verification plan\n"
            "6) Diff vs previous failed attempt"
        )
    return "\n\n".join(sections)


def _load_snapshot_safe(
    snapshot_path: str | None,
    out_dir: str,
    tool_name: str,
    *,
    raw: bool = False,
) -> dict[str, Any] | str:
    """Load a snapshot, returning an error message string on failure."""
    try:
        return (
            load_snapshot_raw(snapshot_path, out_dir)
            if raw
            else load_snapshot(snapshot_path, out_dir)
        )
    except FileNotFoundError:
        return (
            f"No snapshots found in {out_dir}. "
            "Run your tests with llmdebug installed to capture snapshots."
        )
    except (ValueError, ImportError, json.JSONDecodeError, OSError) as e:
        return f"Error loading snapshot: {type(e).__name__}: {e}"
    except Exception as e:
        logger.error("Unexpected error in %s: %s: %s", tool_name, type(e).__name__, e)
        return f"Error loading snapshot: {type(e).__name__}: {e}"


# ---------------------------------------------------------------------------
# MCP Tools
# ---------------------------------------------------------------------------


@mcp.tool()
def llmdebug_diagnose(
    out_dir: str = ".llmdebug",
    detail: str = "context",
    gate_mode: str = "",
    exploratory: bool = False,
) -> str:
    """Get a concise diagnosis of the latest test failure.

    Returns a structured summary with: exception type/message,
    error category/suggestion, crash frame with source and locals,
    repro command, and git context. Optimized for LLM consumption.

    Args:
        out_dir: Snapshot directory (default: .llmdebug)
        detail: Detail level — "crash" (exception + crash frame only),
                "full" (+ all frames), or "context" (+ repro/git, default)
        gate_mode: RCA gate mode override: off|soft|strict (default from env or soft)
        exploratory: Mark this step as exploratory (used by strict gate override logic)
    """
    result = _load_snapshot_safe(None, out_dir, "llmdebug_diagnose")
    if isinstance(result, str):
        return result
    snapshot = result

    sections: list[str] = []

    # Exception (always included)
    exc = snapshot.get("exception", {})
    exc_type = exc.get("type", "UnknownError")
    exc_msg = exc.get("message", "")
    sections.append(f"## Exception\n{exc_type}: {exc_msg}")

    # Hint (error category — always included)
    ecat = exc.get("error_category", {})
    if ecat:
        cat = ecat.get("category", "")
        sug = ecat.get("suggestion", "")
        hint_parts: list[str] = []
        if cat:
            hint_parts.append(f"Category: {cat}")
        if sug:
            hint_parts.append(f"Suggestion: {sug}")
        if hint_parts:
            sections.append("## Hint\n" + "\n".join(hint_parts))

    # Crash frame (always included)
    frames_obj = snapshot.get("frames", [])
    frames = frames_obj if isinstance(frames_obj, list) else []
    if frames:
        crash_idx = coerce_crash_frame_index(snapshot)
        crash_frame = frames[crash_idx]
        if isinstance(crash_frame, dict):
            sections.append(
                f"## Crash Frame (#{crash_idx})\n"
                + _format_frame_text(crash_frame, crash_idx, True)
            )

        # Additional frames only at "full" or "context" level
        if detail in ("full", "context") and len(frames) > 1:
            for i, frame in enumerate(frames):
                if i == crash_idx or not isinstance(frame, dict):
                    continue
                sections.append(f"## Frame #{i}\n" + _format_frame_text(frame, i, False))
        elif len(frames) > 1:
            sections.append(f"({len(frames) - 1} more frame(s) — use llmdebug_frame to inspect)")

    # Context-level sections: repro, git, env
    if detail in ("full", "context"):
        pt = snapshot.get("pytest", {})
        repro = pt.get("repro")
        if repro and isinstance(repro, list):
            cmd_parts = []
            for r in repro:
                if r.endswith("/python3") or r.endswith("/python"):
                    cmd_parts.append("python")
                else:
                    cmd_parts.append(r)
            sections.append(f"## Repro\n{' '.join(cmd_parts)}")

        git = snapshot.get("git", {})
        if git:
            commit = git.get("commit", "")
            branch = git.get("branch", "")
            dirty = " *dirty*" if git.get("dirty") else ""
            if commit:
                sections.append(f"## Git\n{commit} {branch}{dirty}")

    body = "\n\n".join(sections)
    rca_record = _advance_rca_with_snapshot_refs(
        out_dir=out_dir,
        snapshot=snapshot,
        action="bind_evidence",
        gate_mode=gate_mode,
        exploratory=exploratory,
    )
    return _format_tool_response(body, rca_record=rca_record)


@mcp.tool()
def llmdebug_show(
    out_dir: str = ".llmdebug",
    snapshot_path: str = "",
    detail: str = "context",
    raw_session: bool = False,
    with_rca: bool = False,
    gate_mode: str = "",
    exploratory: bool = False,
) -> str:
    """Show the debug snapshot as expanded JSON.

    Returns snapshot data with all frames, locals, and metadata.
    Use llmdebug_diagnose first for a concise summary,
    then llmdebug_show if you need the full picture.

    Args:
        out_dir: Snapshot directory (default: .llmdebug)
        snapshot_path: Path to a specific snapshot file. If empty, uses latest.
        detail: Detail level — "crash", "full", or "context" (default, everything)
        raw_session: Return raw DebugSession envelope when available.
        with_rca: Return JSON envelope with snapshot + RCA block.
        gate_mode: RCA gate mode override: off|soft|strict.
        exploratory: Mark this step as exploratory.
    """
    result = _load_snapshot_safe(
        snapshot_path or None,
        out_dir,
        "llmdebug_show",
        raw=raw_session,
    )
    if isinstance(result, str):
        return result
    snapshot = result

    # Apply detail filter
    if not raw_session and detail in ("crash", "full", "context"):
        try:
            from .detail_filter import filter_snapshot

            snapshot = filter_snapshot(snapshot, detail)  # type: ignore[arg-type]
        except Exception as e:  # nosec B110 — graceful filter fallback
            logger.warning(
                "filter_snapshot(%r) failed, returning unfiltered: %s: %s",
                detail,
                type(e).__name__,
                e,
            )

    if not with_rca:
        return json.dumps(snapshot, indent=2, ensure_ascii=False, default=str)

    normalized_snapshot = snapshot
    if raw_session:
        try:
            from ._debug_session import normalize_snapshot_view

            normalized_snapshot = normalize_snapshot_view(snapshot)
        except Exception as e:
            logger.warning(
                "normalize_snapshot_view failed, using raw: %s: %s",
                type(e).__name__,
                e,
            )
            normalized_snapshot = snapshot

    rca_record = _advance_rca_with_snapshot_refs(
        out_dir=out_dir,
        snapshot=normalized_snapshot if isinstance(normalized_snapshot, dict) else None,
        action="bind_evidence",
        gate_mode=gate_mode,
        exploratory=exploratory,
    )
    payload = {
        "snapshot": snapshot,
        "rca": rca_record,
    }
    return json.dumps(payload, indent=2, ensure_ascii=False, default=str)


@mcp.tool()
def llmdebug_list(out_dir: str = ".llmdebug", limit: int = 10) -> str:
    """List available debug snapshots, newest first.

    Returns snapshot metadata: timestamp, name, exception type,
    message, and file location. Use snapshot_path from the results
    with llmdebug_show to inspect a specific snapshot.

    Args:
        out_dir: Snapshot directory (default: .llmdebug)
        limit: Maximum number of snapshots to return (default: 10)
    """
    try:
        entries = find_all_snapshots(out_dir)
    except OSError as e:
        return f"Error scanning snapshots: {type(e).__name__}: {e}"
    except Exception as e:
        logger.error("Unexpected error in llmdebug_list: %s: %s", type(e).__name__, e)
        return f"Error scanning snapshots: {type(e).__name__}: {e}"

    if not entries:
        return f"No snapshots found in {out_dir}."

    entries = entries[:limit]

    lines: list[str] = [f"Found {len(entries)} snapshot(s) in {out_dir}:\n"]
    for i, entry in enumerate(entries, 1):
        from datetime import datetime, timezone

        ts = datetime.fromtimestamp(entry["mtime"], tz=timezone.utc).strftime("%Y-%m-%d %H:%M:%S")
        loc = f"{entry['file']}:{entry['line']}"
        msg = entry.get("exc_message", "")
        msg_part = f" — {msg}" if msg else ""
        lines.append(f"  {i}. [{ts}] {entry['name']}  {entry['exc_type']}{msg_part}  ({loc})")
        lines.append(f"     path: {entry['path']}")

    return "\n".join(lines)


@mcp.tool()
def llmdebug_frame(
    index: int = 0,
    out_dir: str = ".llmdebug",
    snapshot_path: str = "",
    gate_mode: str = "",
    exploratory: bool = False,
) -> str:
    """Show a specific stack frame in detail.

    Returns source code, all local variables with types and values,
    and function arguments. Frame 0 is the crash site. Use higher
    indices to inspect callers.

    Args:
        index: Frame index (0 = crash site, higher = callers)
        out_dir: Snapshot directory (default: .llmdebug)
        snapshot_path: Path to a specific snapshot file. If empty, uses latest.
        gate_mode: RCA gate mode override: off|soft|strict.
        exploratory: Mark this step as exploratory.
    """
    result = _load_snapshot_safe(snapshot_path or None, out_dir, "llmdebug_frame")
    if isinstance(result, str):
        return result
    snapshot = result

    frames_obj = snapshot.get("frames", [])
    frames = frames_obj if isinstance(frames_obj, list) else []
    if not frames:
        return "No frames in snapshot."

    if index < 0 or index >= len(frames):
        return (
            f"Frame index {index} out of range. "
            f"Available frames: 0-{len(frames) - 1} ({len(frames)} total)."
        )

    frame = frames[index]
    if not isinstance(frame, dict):
        return f"Frame index {index} is malformed in snapshot."
    crash_idx = coerce_crash_frame_index(snapshot)
    is_crash = index == crash_idx

    body = _format_frame_text(frame, index, is_crash)
    rca_record = _advance_rca_with_snapshot_refs(
        out_dir=out_dir,
        snapshot=snapshot,
        action="bind_evidence",
        gate_mode=gate_mode,
        exploratory=exploratory,
    )
    return _format_tool_response(body, rca_record=rca_record)


@mcp.tool()
def llmdebug_git_context(
    out_dir: str = ".llmdebug",
    snapshot_path: str = "",
    max_commits: int = 5,
    gate_mode: str = "",
    exploratory: bool = False,
) -> str:
    """Show on-demand enhanced git metadata for a snapshot.

    Returns metadata-only context:
    - crash-line blame metadata
    - recent commit metadata
    - crash-file diffstat metadata
    """
    result = _load_snapshot_safe(snapshot_path or None, out_dir, "llmdebug_git_context")
    if isinstance(result, str):
        return result
    snapshot = result

    try:
        max_commits_parsed = int(max_commits)
    except (TypeError, ValueError):
        return f"Invalid max_commits value: {max_commits!r}. Expected an integer between 1 and 50."
    max_commits = max(1, min(max_commits_parsed, 50))
    enhanced = get_enhanced_git_context(snapshot, out_dir=out_dir, max_commits=max_commits)
    if enhanced is None:
        body = "Enhanced git context unavailable (not in a git repository or git is unavailable)."
        rca_record = _advance_rca_with_snapshot_refs(
            out_dir=out_dir,
            snapshot=snapshot,
            action="bind_evidence",
            gate_mode=gate_mode,
            exploratory=exploratory,
        )
        return _format_tool_response(body, rca_record=rca_record)

    evidence_refs: list[str] = []
    head = enhanced.get("head")
    if isinstance(head, dict):
        commit = str(head.get("commit") or "").strip()
        branch = str(head.get("branch") or "").strip()
        if commit:
            evidence_refs.append(f"git_head_commit:{commit}")
        if branch:
            evidence_refs.append(f"git_branch:{branch}")

    crash_file = enhanced.get("crash_file")
    if isinstance(crash_file, dict):
        file_rel = str(crash_file.get("file_rel") or "").strip()
        if file_rel:
            evidence_refs.append(f"git_crash_file:{file_rel}")
        blame = crash_file.get("blame")
        if isinstance(blame, dict):
            blame_commit = str(blame.get("commit") or "").strip()
            if blame_commit:
                evidence_refs.append(f"git_blame_commit:{blame_commit}")

    body = format_enhanced_git_context_text(enhanced)
    rca_record = _advance_rca_with_snapshot_refs(
        out_dir=out_dir,
        snapshot=snapshot,
        action="bind_evidence",
        gate_mode=gate_mode,
        extra_evidence_refs=evidence_refs,
        exploratory=exploratory,
    )
    return _format_tool_response(body, rca_record=rca_record)


@mcp.tool()
def llmdebug_diff(
    old_ref: str = "previous",
    new_ref: str = "latest",
    out_dir: str = ".llmdebug",
    gate_mode: str = "",
    exploratory: bool = False,
) -> str:
    """Compare two snapshots to show what changed between debugging iterations.

    Returns changes in: exception type/message, crash location,
    local variable values, array shapes, and git context.
    Default: compares the two most recent snapshots.

    Args:
        old_ref: Older snapshot — "latest", "previous", "#N" (list index), or file path.
        new_ref: Newer snapshot — same format as old_ref.
        out_dir: Snapshot directory (default: .llmdebug)
        gate_mode: RCA gate mode override: off|soft|strict.
        exploratory: Mark this step as exploratory.
    """
    try:
        old = resolve_snapshot_ref(old_ref, out_dir)
        new = resolve_snapshot_ref(new_ref, out_dir)
    except (FileNotFoundError, ValueError) as e:
        return str(e)
    except (ImportError, json.JSONDecodeError, OSError) as e:
        return f"Error loading snapshots: {type(e).__name__}: {e}"
    except Exception as e:
        logger.error("Unexpected error in llmdebug_diff: %s: %s", type(e).__name__, e)
        return f"Error loading snapshots: {type(e).__name__}: {e}"

    result = diff_snapshots(old, new)
    structured = structured_diff_summary(result)
    verification_status = "fail" if result.failure_signature_same else "inconclusive"
    verification_result = {
        "status": verification_status,
        "checks": ["llmdebug_diff"],
    }
    previous_refs = [f"previous:{ref}" for ref in _extract_evidence_refs(old)]
    rca_record = _advance_rca_with_snapshot_refs(
        out_dir=out_dir,
        snapshot=new,
        action="verify",
        gate_mode=gate_mode,
        extra_evidence_refs=previous_refs,
        verification_result=verification_result,
        proposed_change_summary=(
            "Compared previous and latest failures; inspected exception/location/locals deltas."
        ),
        exploratory=exploratory,
    )
    body = format_diff_text(result)
    return _format_tool_response(body, rca_record=rca_record, structured_delta=structured)


@mcp.tool()
def llmdebug_hypothesize(
    out_dir: str = ".llmdebug",
    snapshot_path: str = "",
    gate_mode: str = "",
    exploratory: bool = False,
) -> str:
    """Generate ranked debugging hypotheses from a snapshot.

    Pattern-matches on exception type, local variable values, array shapes,
    and metadata to produce ranked hypotheses with confidence scores,
    evidence, and suggested next steps.

    Args:
        out_dir: Snapshot directory (default: .llmdebug)
        snapshot_path: Path to a specific snapshot file. If empty, uses latest.
        gate_mode: RCA gate mode override: off|soft|strict.
        exploratory: Mark this step as exploratory.
    """
    result = _load_snapshot_safe(snapshot_path or None, out_dir, "llmdebug_hypothesize")
    if isinstance(result, str):
        return result
    snapshot = result

    from .hypotheses import format_hypotheses_text, generate_hypotheses

    hypotheses = generate_hypotheses(snapshot)
    top_hypothesis = hypotheses[0].description if hypotheses else ""
    experiment_plan = hypotheses[0].suggestion if hypotheses else ""
    hypothesis_refs = list(hypotheses[0].evidence[:3]) if hypotheses else []
    rca_record = _advance_rca_with_snapshot_refs(
        out_dir=out_dir,
        snapshot=snapshot,
        action="hypothesize",
        gate_mode=gate_mode,
        extra_evidence_refs=hypothesis_refs,
        hypothesis=top_hypothesis,
        experiment_plan=experiment_plan,
        exploratory=exploratory,
    )
    body = format_hypotheses_text(hypotheses)
    return _format_tool_response(body, rca_record=rca_record)


@mcp.tool()
def llmdebug_rca_status(
    out_dir: str = ".llmdebug",
    session_id: str = "",
    snapshot_path: str = "",
) -> str:
    """Return latest RCA state for a session (or latest overall if session unspecified)."""
    resolved_session = session_id.strip()
    if not resolved_session and snapshot_path:
        try:
            snap = load_snapshot(snapshot_path, out_dir)
            resolved_session = derive_session_id(snap, out_dir=out_dir, fallback="mcp")
        except Exception as e:
            logger.warning("session ID resolution failed: %s: %s", type(e).__name__, e)
            resolved_session = ""
    elif not resolved_session:
        try:
            snap = load_snapshot(None, out_dir)
            resolved_session = derive_session_id(snap, out_dir=out_dir, fallback="mcp")
        except Exception as e:
            logger.warning("session ID resolution failed: %s: %s", type(e).__name__, e)
            resolved_session = ""

    status = get_rca_status(out_dir=out_dir, session_id=resolved_session or None)
    if status is None:
        if resolved_session:
            return f"No RCA attempts found for session {resolved_session!r} in {out_dir}."
        return f"No RCA attempts found in {out_dir}."
    return json.dumps(status, indent=2, ensure_ascii=False)


@mcp.tool()
def llmdebug_rca_history(
    out_dir: str = ".llmdebug",
    session_id: str = "",
    limit: int = 20,
) -> str:
    """Return RCA history (newest-first) for a session or globally."""
    history = get_rca_history(out_dir=out_dir, session_id=session_id.strip() or None, limit=limit)
    if not history:
        if session_id.strip():
            return f"No RCA attempts found for session {session_id!r} in {out_dir}."
        return f"No RCA attempts found in {out_dir}."
    return json.dumps(history, indent=2, ensure_ascii=False)


@mcp.tool()
def llmdebug_rca_advance(
    action: str,
    out_dir: str = ".llmdebug",
    session_id: str = "",
    snapshot_path: str = "",
    failure_signature: str = "",
    evidence_refs: str = "",
    hypothesis: str = "",
    experiment_plan: str = "",
    proposed_change_summary: str = "",
    verification_status: str = "",
    verification_checks: str = "",
    gate_mode: str = "",
    exploratory: bool = False,
    override_reason: str = "",
) -> str:
    """Manually advance RCA state machine for custom agent workflows."""
    snapshot: dict[str, Any] | None = None
    if snapshot_path:
        try:
            snapshot = load_snapshot(snapshot_path, out_dir)
        except Exception as exc:
            return f"Error loading snapshot: {type(exc).__name__}: {exc}"
    elif not session_id or not failure_signature:
        try:
            snapshot = load_snapshot(None, out_dir)
        except Exception:
            snapshot = None

    resolved_session = session_id.strip()
    if not resolved_session:
        resolved_session = derive_session_id(snapshot, out_dir=out_dir, fallback="manual")
    signature = failure_signature.strip() or build_failure_signature(snapshot)

    refs = [item.strip() for item in evidence_refs.split(",") if item.strip()]
    verification_result: dict[str, Any] | None = None
    if verification_status.strip():
        verification_result = {
            "status": verification_status.strip(),
            "checks": [item.strip() for item in verification_checks.split(",") if item.strip()],
        }

    try:
        record = advance_rca_attempt(
            out_dir=out_dir,
            session_id=resolved_session,
            action=action,
            failure_signature=signature,
            evidence_refs=refs,
            hypothesis=hypothesis.strip() or None,
            experiment_plan=experiment_plan.strip() or None,
            proposed_change_summary=proposed_change_summary.strip() or None,
            verification_result=verification_result,
            gate_mode=_effective_gate_mode(gate_mode),
            exploratory=exploratory,
            override_reason=override_reason.strip() or None,
        )
    except Exception as exc:
        return f"Error updating RCA state: {type(exc).__name__}: {exc}"
    return json.dumps(record, indent=2, ensure_ascii=False)


# ---------------------------------------------------------------------------
# Entry point
# ---------------------------------------------------------------------------


def main() -> None:
    """Run the llmdebug MCP server."""
    logging.basicConfig(
        stream=sys.stderr,
        level=logging.WARNING,
        format="%(name)s: %(message)s",
    )
    mcp.run(transport="stdio")


if __name__ == "__main__":
    main()



def main():
    cards = list(map(int, input().split()))
    assert len(cards) == 5

    cards.sort()
    if (
        cards[0] == cards[1]
        and cards[0] == cards[2]
        and cards[0] != cards[3]
        and cards[3] == cards[4]
    ):
        print("Yes")
    else:
        print("No")


if __name__ == "__main__":
    main()



import subprocess
import sys


def test_condefects_0():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='5 6 3\n1 2 2\n2 3 3\n1 3 6\n2 4 5\n4 5 9\n3 5 8\n1 3 1\n3 4 7\n3 5 7\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = 'Yes\nNo\nYes\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_1():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='2 3 2\n1 2 100\n1 2 1000000000\n1 1 1\n1 2 2\n1 1 5\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = 'Yes\nNo\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )



# Accepted. From the official editorial.
K = int(input())
Sx, Sy = [int(x) for x in input().split()]
Tx, Ty = [int(x) for x in input().split()]
# Kずつ右上にずらしておく。
Sx += K
Sy += K
Tx += K
Ty += K
# 上界を入れておく。
dist = max(Tx, Sx) - min(Tx, Sx) + max(Ty, Sy) - min(Ty, Sy)
# 1 < Kのとき、大タイルを経由する移動を考える。
if 1 < K:
    large_start = []  # はじめに到達する大タイルの候補。
    if ((Sx // K) ^ (Sy // K)) & 1:  # 開始位置が大タイルなら。
        large_start.append((Sx // K, Sy // K, 0))  # その大タイル。
    else:  # そうでなければ、四方向いずれかで最も近い大タイルが候補。
        large_start.append((Sx // K - 1, Sy // K, 1 + Sx % K))  # 左。
        large_start.append((Sx // K + 1, Sy // K, K - Sx % K))  # 右。
        large_start.append((Sx // K, Sy // K - 1, 1 + Sy % K))  # 下。
        large_start.append((Sx // K, Sy // K + 1, K - Sy % K))  # 上。
    large_goal = []  # 最後に到達する大タイルの候補。
    if ((Tx // K) ^ (Ty // K)) & 1:  # 終了位置が大タイルなら。
        large_goal.append((Tx // K, Ty // K, 0))  # その大タイル。
    else:  # そうでなければ、四方向いずれかで最も近い大タイルが候補。
        large_goal.append((Tx // K - 1, Ty // K, 1 + Tx % K))  # 左。
        large_goal.append((Tx // K + 1, Ty // K, K - Tx % K))  # 右。
        large_goal.append((Tx // K, Ty // K - 1, 1 + Ty % K))  # 下。
        large_goal.append((Tx // K, Ty // K + 1, K - Ty % K))  # 上。
    # K = 2かどうかで場合分け。
    if K == 2:
        for x, y, d1 in large_start:
            for z, w, d2 in large_goal:
                x_diff = max(x, z) - min(x, z)
                y_diff = max(y, w) - min(y, w)
                dist = min(
                    dist,
                    d1 + d2 + x_diff + y_diff + (max(x_diff, y_diff) - min(x_diff, y_diff)) / 2,
                )
    else:
        for x, y, d1 in large_start:
            for z, w, d2 in large_goal:
                dist = min(
                    dist,
                    d1
                    + d2
                    + max(x + y, z + w)
                    - min(x + y, z + w)
                    + max(x + w, z + y)
                    - min(x + w, z + y),
                )
print(dist)



N = int(input())
S = list(str(input()))
C = list(map(int, input().split()))


A = []
B = []

for i in range(N + 1):
    if i == 0:
        A.append(0)
        B.append(0)
    elif int(S[i - 1]) == i % 2:
        A.append(A[i - 1] + C[i - 1])
        B.append(B[i - 1])
    else:
        A.append(A[i - 1])
        B.append(B[i - 1] + C[i - 1])

ans = 10**20
for j in range(N):
    ans = min(ans, A[j] - A[0] + B[N] - B[j], B[j] - B[0] + A[N] - A[j])

print(ans)



W, B = map(int, input().split())
wb = "wbwbwwbwbwbw"
wball = wb * (200 // len(wb) + 4)
n = len(wball)

for i in range(n - (W + B)):
    if wball[i : (W + B + 1)].count("w") == W and wball[i : (W + B + 1)].count("b") == B:
        print("Yes")
        exit()
print("No")



from typing import List, Optional, Tuple, Dict, Set
from collections import defaultdict, Counter, deque
import heapq
import math
import bisect
import itertools
import functools
class Solution:
def thirdMax(self, nums) -> int:
    return self.max(list(set(nums))) if len(list(set(nums)))<3 else sorted(list(set(nums)))[-3]



from collections import deque

n = int(input())
input_list = [int(x) for x in input().split(" ")]


tfe = deque()

temp = None
for x in input_list:
    temp = x
    if tfe and tfe[-1] == temp:
        while tfe and tfe[-1] == temp:
            temp += 1
            tfe.pop()
    else:
        tfe.append(temp)



def solve():
    n = int(input())
    s = input()
    for k in range(1, n):
        flag = True
        for i in range(min(k, n - k)):
            if s[i] < s[k + i]:
                print("Yes")
                return
            elif s[i] == s[k + i]:
                continue
            else:
                flag = False
                break
        if flag and k <= n - k:
            print("Yes")
            return
    print("No")


for i in range(int(input())):
    solve()



from collections import deque


def solve():
    n, m = map(int, input().split())
    c = list(map(int, input().split()))
    g = [[] for _ in range(n)]
    for _ in range(m):
        u, v = map(int, input().split())
        u -= 1
        v -= 1
        g[u].append(v)
        g[v].append(u)

    if c[0] == c[n - 1]:
        print(-1)
        return

    visited = set()
    visited.add((0, n - 1))
    dq = deque()
    dq.append((0, n - 1, 0))
    while dq:
        p1, p2, cnt = dq.popleft()
        if (p1, p2) == (n - 1, 0):
            print(cnt)
            return
        for p1nex in g[p1]:
            for p2nex in g[p2]:
                if c[p1nex] != c[p2nex] and (p1nex, p2nex) not in visited:
                    dq.append((p1nex, p2nex, cnt + 1))
                    visited.add((p1nex, p2nex))


T = int(input())
for _ in range(T):
    solve()



from collections import deque


n = int(input())
s = list(input())
t = list(input())
if sorted(s) != sorted(t):
    print(-1)
else:
    ok = n
    ng = 0
    while True:
        # print(ng,ok)
        k = (ok + ng) // 2
        j = 0
        p = s[k:]
        g = 0
        if k == n:
            g = 1
        else:
            for i in range(n):
                if t[i] == p[j]:
                    j += 1
                if j == len(p):
                    g += 1
                    break
            # p=s[:n-k]
            # j=0
            # for i in range(n):
            #     if t[i]==p[j]:
            #         j+=1
            #     if j==len(p):
            #         g+=1
            #         break
        if g == 1:
            ok = k
        else:
            ng = k
        if abs(ok - ng) <= 1:
            # print(ng,ok)
            print((ok + ng) // 2 + 1)
            break



from collections import deque
import heapq

m = int(input())
v = [tuple(map(int, input().split())) for i in range(m)]
p = list(map(int, input().split()))
p = list(set(p) ^ set(range(1, 10))) + p
p = [i - 1 for i in p]
p = tuple([p.index(i) for i in range(9)])
g = {i: [] for i in range(9)}
for i in range(m):
    a, b = v[i]
    g[a - 1] += [b - 1]
    g[b - 1] += [a - 1]

start = p
visited = set()
visited.add(p)
dist = {}
dist[p] = 0
que = deque([start])

while len(que):
    v = que.popleft()
    nvs = []
    ind = v.index(0)
    for i in g[ind]:
        temp = list(v)
        temp[i], temp[ind] = temp[ind], temp[i]
        nvs += [tuple(temp)]
    for nv in nvs:
        if nv in visited:
            continue
        visited.add(nv)
        que.append(nv)
        dist[nv] = dist[v] + 1
if (1, 2, 3, 4, 5, 6, 7, 8, 0) in dist:
    print(dist[(1, 2, 3, 4, 5, 6, 7, 8, 0)])
else:
    print(0)



T = int(input())
for _ in range(T):
    N, K = map(int, input().split())
    P = [0, 0] + list(map(int, input().split()))
    A = [0] + list(map(int, input().split()))
    G = {i: [] for i in range(1, N + 1)}
    for i in range(2, N + 1):
        G[P[i]].append(i)
    lack = {i: [True, 0, set()] for i in range(1, N + 1)}
    B = set(range(K))

    def dfs(x):
        if A[x] == K:
            lack[x][0] = False
        if len(G[x]) == 0:
            if A[x] >= 0:
                lack[x][2] = B - set([A[x]])
            else:
                lack[x][1] += 1
                lack[x][2] = B
        else:
            lack[x][2] = B
            for y in G[x]:
                dfs(y)
                lack[x][0] = lack[x][0] and lack[y][0]
                lack[x][1] += lack[y][1]
                lack[x][2] = lack[x][2] & lack[y][2]
            if A[x] == -1:
                lack[x][1] += 1

    dfs(1)
    flag = "Bob"
    for i in range(1, N + 1):
        if not lack[i][0]:
            continue
        if (len(lack[i][2]) == 0 and lack[i][1] <= 1) or (len(lack[i][2]) == 1 and lack[i][1] == 1):
            flag = "Alice"
            break
    print(flag)



class RingPicker:
    def __init__(self) -> None:
        self._index = 0

    def pick(self, items: list[str]) -> str:
        value = items[self._index]
        self._index += 1
        return value



import subprocess
import sys


def test_condefects_0():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='3\n7 2\n1 6\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '5\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_1():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='1\n41 42\n13 56\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '42\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_2():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='100\n100 99\n199 1\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '0\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_3():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='96929423\n5105216413055191 10822465733465225\n1543712011036057 14412421458305526\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '79154049\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )


def test_condefects_4():
    proc = subprocess.run(
        [sys.executable, "buggy.py"],
        input='2\n6865455395347484 9327767733799873\n7394420205524045 9558146353019185\n',
        capture_output=True,
        text=True,
        timeout=10,
    )
    expected = '454318262437249\n'
    actual = proc.stdout.strip()
    assert actual == expected.strip(), (
        f"Expected {expected.strip()!r}, got {actual!r}"
    )
